# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import markuptools
from abjad.tools import mathtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject
from abjad.tools.topleveltools.new import new


class MetricModulation(AbjadValueObject):
    r'''Metric modulation.

    ..  container:: example

        **Example 1.** With notes:

        ::

            >>> metric_modulation = indicatortools.MetricModulation(
            ...     left_rhythm=Note("c'4"),
            ...     right_rhythm=Note("c'4."),
            ...     )

        ::

            >>> show(metric_modulation) # doctest: +SKIP

        ..  doctest::

            >>> print(format(metric_modulation, 'lilypond'))
            \markup {
                \scale
                    #'(0.75 . 0.75)
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    c'4
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                =
                \hspace
                    #-0.5
                \scale
                    #'(0.75 . 0.75)
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    c'4.
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                }

    ..  container:: example

        **Example 2.** With tuplets:

        ::

            >>> metric_modulation = indicatortools.MetricModulation(
            ...     left_rhythm=Tuplet((4, 5), "c'4"),
            ...     right_rhythm=Note("c'4"),
            ...     )

        ::

            >>> show(metric_modulation) # doctest: +SKIP

        ..  doctest::

            >>> print(format(metric_modulation, 'lilypond'))
            \markup {
                \scale
                    #'(0.75 . 0.75)
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    \tweak edge-height #'(0.7 . 0)
                                    \times 4/5 {
                                        c'4
                                    }
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                =
                \hspace
                    #-0.5
                \scale
                    #'(0.75 . 0.75)
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    c'4
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                }

    ..  container:: example

        **Example 3.** With tuplets again:

        ::

            >>> metric_modulation = indicatortools.MetricModulation(
            ...     left_rhythm=Note("c16."),
            ...     right_rhythm=Tuplet((2, 3), "c8"),
            ...     )

        ::

            >>> show(metric_modulation) # doctest: +SKIP

        ..  doctest::

            >>> print(format(metric_modulation, 'lilypond'))
            \markup {
                \scale
                    #'(0.75 . 0.75)
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    c16.
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                =
                \hspace
                    #-0.5
                \scale
                    #'(0.75 . 0.75)
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    \tweak edge-height #'(0.7 . 0)
                                    \times 2/3 {
                                        c8
                                    }
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                }

    ..  container:: example

        **Example 4.** With ties:

        ::

            >>> notes = scoretools.make_notes([0], [Duration(5, 16)])
            >>> metric_modulation = indicatortools.MetricModulation(
            ...     left_rhythm=Note("c'4"),
            ...     right_rhythm=notes,
            ...     )

        ::

            >>> show(metric_modulation) # doctest: +SKIP

        ..  doctest::

            >>> print(format(metric_modulation, 'lilypond'))
            \markup {
                \scale
                    #'(0.75 . 0.75)
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    c'4
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                =
                \hspace
                    #-0.5
                \scale
                    #'(0.75 . 0.75)
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    c'4 ~
                                    c'16
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                }

    ..  container:: example

        **Example 5.** With ties and tuplets:

        ::

            >>> notes = scoretools.make_notes([0], [Duration(5, 16)])
            >>> tuplet = Tuplet((2, 3), notes)
            >>> metric_modulation = indicatortools.MetricModulation(
            ...     left_rhythm=Note("c'4"),
            ...     right_rhythm=tuplet,
            ...     )

        ::

            >>> show(metric_modulation) # doctest: +SKIP

        ..  doctest::

            >>> print(format(metric_modulation, 'lilypond'))
            \markup {
                \scale
                    #'(0.75 . 0.75)
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    c'4
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                =
                \hspace
                    #-0.5
                \scale
                    #'(0.75 . 0.75)
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    \tweak edge-height #'(0.7 . 0)
                                    \times 2/3 {
                                        c'4 ~
                                        c'16
                                    }
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                }


    ..  container:: example

        **Example 6.** Attach metric modulations to generate score output:

        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4 e'4 d'4")
            >>> attach(TimeSignature((3, 4)), staff)
            >>> score = Score([staff])

        ::

            >>> metric_modulation = indicatortools.MetricModulation(
            ...     left_rhythm=Note("c4"),
            ...     right_rhythm=Note("c8."),
            ...     )
            >>> attach(metric_modulation, staff[3])
            >>> override(staff).text_script.staff_padding = 2.5

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff \with {
                    \override TextScript.staff-padding = #2.5
                } {
                    \time 3/4
                    c'4
                    d'4
                    e'4
                    f'4
                        ^ \markup {
                            \scale
                                #'(0.75 . 0.75)
                                \score
                                    {
                                        \new Score \with {
                                            \override SpacingSpanner.spacing-increment = #0.5
                                            proportionalNotationDuration = ##f
                                        } <<
                                            \new RhythmicStaff \with {
                                                \remove Time_signature_engraver
                                                \remove Staff_symbol_engraver
                                                \override Stem.direction = #up
                                                \override Stem.length = #5
                                                \override TupletBracket.bracket-visibility = ##t
                                                \override TupletBracket.direction = #up
                                                \override TupletBracket.padding = #1.25
                                                \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                                \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                                tupletFullLength = ##t
                                            } {
                                                c4
                                            }
                                        >>
                                        \layout {
                                            indent = #0
                                            ragged-right = ##t
                                        }
                                    }
                            =
                            \hspace
                                #-0.5
                            \scale
                                #'(0.75 . 0.75)
                                \score
                                    {
                                        \new Score \with {
                                            \override SpacingSpanner.spacing-increment = #0.5
                                            proportionalNotationDuration = ##f
                                        } <<
                                            \new RhythmicStaff \with {
                                                \remove Time_signature_engraver
                                                \remove Staff_symbol_engraver
                                                \override Stem.direction = #up
                                                \override Stem.length = #5
                                                \override TupletBracket.bracket-visibility = ##t
                                                \override TupletBracket.direction = #up
                                                \override TupletBracket.padding = #1.25
                                                \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                                \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                                tupletFullLength = ##t
                                            } {
                                                c8.
                                            }
                                        >>
                                        \layout {
                                            indent = #0
                                            ragged-right = ##t
                                        }
                                    }
                            }
                    e'4
                    d'4
                }
            >>

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_left_markup',
        '_left_rhythm',
        '_right_markup',
        '_right_rhythm',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        left_rhythm=None,
        right_rhythm=None,
        left_markup=None,
        right_markup=None,
        ):
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        # TODO: make default scope work
        #self._default_scope = scoretools.Score
        self._default_scope = None
        left_rhythm = left_rhythm or scoretools.Note('c4')
        right_rhythm = right_rhythm or scoretools.Note('c4')
        left_rhythm = self._initialize_rhythm(left_rhythm)
        self._left_rhythm = left_rhythm
        right_rhythm = self._initialize_rhythm(right_rhythm)
        self._right_rhythm = right_rhythm
        self._right_rhythm = right_rhythm
        if left_markup is not None:
            assert isinstance(left_markup, markuptools.Markup)
        self._left_markup = left_markup
        if right_markup is not None:
            assert isinstance(right_markup, markuptools.Markup)
        self._right_markup = right_markup

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true `argument` is another metric modulation with the same ratio as
        this metric modulation. Otherwise false.

        ..  container:: example

            ::

                >>> metric_modulation_1 = indicatortools.MetricModulation(
                ...     left_rhythm=Note("c'4"),
                ...     right_rhythm=Note("c'4."),
                ...     )
                >>> metric_modulation_2 = indicatortools.MetricModulation(
                ...     left_rhythm=Tuplet((2, 3), [Note("c'4")]),
                ...     right_rhythm=Note("c'4"),
                ...     )
                >>> notes = scoretools.make_notes([0], [Duration(5, 16)])
                >>> metric_modulation_3 = indicatortools.MetricModulation(
                ...     left_rhythm=Note("c'4"),
                ...     right_rhythm=notes,
                ...     )

            ::

                >>> metric_modulation_1.ratio
                Ratio((2, 3))
                >>> metric_modulation_2.ratio
                Ratio((2, 3))
                >>> metric_modulation_3.ratio
                Ratio((4, 5))

            ::

                >>> metric_modulation_1 == metric_modulation_1
                True
                >>> metric_modulation_1 == metric_modulation_2
                True
                >>> metric_modulation_1 == metric_modulation_3
                False

            ::

                >>> metric_modulation_2 == metric_modulation_1
                True
                >>> metric_modulation_2 == metric_modulation_2
                True
                >>> metric_modulation_2 == metric_modulation_3
                False

            ::

                >>> metric_modulation_3 == metric_modulation_1
                False
                >>> metric_modulation_3 == metric_modulation_2
                False
                >>> metric_modulation_3 == metric_modulation_3
                True

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            if self.ratio == argument.ratio:
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats metric modulation.

        ..  container:: example

            ::

                >>> metric_modulation = indicatortools.MetricModulation(
                ...     left_rhythm=Note("c'4"),
                ...     right_rhythm=Note("c'4."),
                ...     )

            ::

                >>> print(format(metric_modulation))
                indicatortools.MetricModulation(
                    left_rhythm=selectiontools.Selection(
                        [
                            scoretools.Note("c'4"),
                            ]
                        ),
                    right_rhythm=selectiontools.Selection(
                        [
                            scoretools.Note("c'4."),
                            ]
                        ),
                    )

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        superclass = super(MetricModulation, self)
        return superclass.__format__(format_specification=format_specification)

    def __hash__(self):
        r'''Hashes metric modulation.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(MetricModulation, self).__hash__()

    def __illustrate__(self):
        r'''Illustrates metric modulation.

        ..  container:: example

            ::

                >>> metric_modulation = indicatortools.MetricModulation(
                ...     left_rhythm=Tuplet((2, 3), "c'4"),
                ...     right_rhythm=Note("c'4."),
                ...     )
                >>> show(metric_modulation) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = metric_modulation.__illustrate__()
                >>> metric_modulation = lilypond_file.items[-1]
                >>> print(format(metric_modulation))
                \markup {
                    \scale
                        #'(0.75 . 0.75)
                        \score
                            {
                                \new Score \with {
                                    \override SpacingSpanner.spacing-increment = #0.5
                                    proportionalNotationDuration = ##f
                                } <<
                                    \new RhythmicStaff \with {
                                        \remove Time_signature_engraver
                                        \remove Staff_symbol_engraver
                                        \override Stem.direction = #up
                                        \override Stem.length = #5
                                        \override TupletBracket.bracket-visibility = ##t
                                        \override TupletBracket.direction = #up
                                        \override TupletBracket.padding = #1.25
                                        \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                        \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                        tupletFullLength = ##t
                                    } {
                                        \tweak edge-height #'(0.7 . 0)
                                        \times 2/3 {
                                            c'4
                                        }
                                    }
                                >>
                                \layout {
                                    indent = #0
                                    ragged-right = ##t
                                }
                            }
                    =
                    \hspace
                        #-0.5
                    \scale
                        #'(0.75 . 0.75)
                        \score
                            {
                                \new Score \with {
                                    \override SpacingSpanner.spacing-increment = #0.5
                                    proportionalNotationDuration = ##f
                                } <<
                                    \new RhythmicStaff \with {
                                        \remove Time_signature_engraver
                                        \remove Staff_symbol_engraver
                                        \override Stem.direction = #up
                                        \override Stem.length = #5
                                        \override TupletBracket.bracket-visibility = ##t
                                        \override TupletBracket.direction = #up
                                        \override TupletBracket.padding = #1.25
                                        \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                        \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                        tupletFullLength = ##t
                                    } {
                                        c'4.
                                    }
                                >>
                                \layout {
                                    indent = #0
                                    ragged-right = ##t
                                }
                            }
                    }

        Returns LilyPond file.
        '''
        import abjad
        lilypond_file = abjad.LilyPondFile.new()
        lilypond_file.header_block.tagline = False
        lilypond_file.items.append(self._get_markup())
        return lilypond_file

    def __str__(self):
        r'''Gets string representation of metric modulation.

        ..  container:: example

            ::

                >>> metric_modulation = indicatortools.MetricModulation(
                ...     left_rhythm=Tuplet((2, 3), [Note("c'4")]),
                ...     right_rhythm=Note("c'4"),
                ...     )

            ::

                >>> print(str(metric_modulation))
                \markup {
                    \scale
                        #'(0.75 . 0.75)
                        \score
                            {
                                \new Score \with {
                                    \override SpacingSpanner.spacing-increment = #0.5
                                    proportionalNotationDuration = ##f
                                } <<
                                    \new RhythmicStaff \with {
                                        \remove Time_signature_engraver
                                        \remove Staff_symbol_engraver
                                        \override Stem.direction = #up
                                        \override Stem.length = #5
                                        \override TupletBracket.bracket-visibility = ##t
                                        \override TupletBracket.direction = #up
                                        \override TupletBracket.padding = #1.25
                                        \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                        \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                        tupletFullLength = ##t
                                    } {
                                        \tweak edge-height #'(0.7 . 0)
                                        \times 2/3 {
                                            c'4
                                        }
                                    }
                                >>
                                \layout {
                                    indent = #0
                                    ragged-right = ##t
                                }
                            }
                    =
                    \hspace
                        #-0.5
                    \scale
                        #'(0.75 . 0.75)
                        \score
                            {
                                \new Score \with {
                                    \override SpacingSpanner.spacing-increment = #0.5
                                    proportionalNotationDuration = ##f
                                } <<
                                    \new RhythmicStaff \with {
                                        \remove Time_signature_engraver
                                        \remove Staff_symbol_engraver
                                        \override Stem.direction = #up
                                        \override Stem.length = #5
                                        \override TupletBracket.bracket-visibility = ##t
                                        \override TupletBracket.direction = #up
                                        \override TupletBracket.padding = #1.25
                                        \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                        \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                        tupletFullLength = ##t
                                    } {
                                        c'4
                                    }
                                >>
                                \layout {
                                    indent = #0
                                    ragged-right = ##t
                                }
                            }
                    }

        Returns string.
        '''
        return str(self._get_markup())

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return str(self)

    ### PRIVATE METHODS ###

    def _get_left_markup(self):
        if self.left_markup is not None:
            return self.left_markup
        markup = durationtools.Duration._to_score_markup(self.left_rhythm)
        return markup

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        markup = self._get_markup()
        markup = new(markup, direction=Up)
        markup_format_pieces = markup._get_format_pieces()
        lilypond_format_bundle.right.markup.extend(markup_format_pieces)
        return lilypond_format_bundle

    def _get_markup(self, music_scale_pair=(0.75, 0.75)):
        assert isinstance(music_scale_pair, (tuple, type(None)))
        left_markup = self._get_left_markup()
        if music_scale_pair:
            left_markup = left_markup.scale(music_scale_pair)
        equal = markuptools.Markup('=')
        right_space = markuptools.Markup.hspace(-0.5)
        right_markup = self._get_right_markup()
        if music_scale_pair:
            right_markup = right_markup.scale(music_scale_pair)
        markup = left_markup + equal + right_space + right_markup
        return markup

    def _get_right_markup(self):
        if self.right_markup is not None:
            return self.right_markup
        markup = durationtools.Duration._to_score_markup(self.right_rhythm)
        return markup

    def _initialize_rhythm(self, rhythm):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        if isinstance(rhythm, scoretools.Component):
            selection = selectiontools.Selection([rhythm])
        elif isinstance(rhythm, selectiontools.Selection):
            selection = rhythm
        else:
            message = 'rhythm must be duration, component or selection: {!r}.'
            message = message.format(rhythm)
            raise TypeError(message)
        assert isinstance(selection, selectiontools.Selection)
        return selection

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of metric modulation.

        ..  container:: example

            ::

                >>> metric_modulation = indicatortools.MetricModulation(
                ...     left_rhythm=Note("c'4"),
                ...     right_rhythm=Note("c'4."),
                ...     )
                >>> metric_modulation.default_scope is None
                True

        .. todo:: Metric modulations should be score-scope.

        Returns none (but should return score).
        '''
        return self._default_scope

    @property
    def left_markup(self):
        r'''Gets left markup of metric modulation.

        ..  container:: example

            ::

                >>> metric_modulation = indicatortools.MetricModulation(
                ...     left_rhythm=Note("c'4"),
                ...     right_rhythm=Note("c'4."),
                ...     )
                >>> metric_modulation.left_markup

        Returns markup or none.
        '''
        return self._left_markup

    @property
    def left_rhythm(self):
        r'''Gets left rhythm of metric modulation.

        ..  container:: example

            ::

                >>> metric_modulation = indicatortools.MetricModulation(
                ...     left_rhythm=Note("c'4"),
                ...     right_rhythm=Note("c'4."),
                ...     )
                >>> metric_modulation.left_rhythm
                Selection([Note("c'4")])

        Returns selection.
        '''
        return self._left_rhythm

    @property
    def ratio(self):
        r'''Gets ratio of metric modulation.

        ..  container:: example

            ::

                >>> metric_modulation = indicatortools.MetricModulation(
                ...     left_rhythm=Tuplet((2, 3), [Note("c'4")]),
                ...     right_rhythm=Note("c'4"),
                ...     )
                >>> metric_modulation.ratio
                Ratio((2, 3))

        Returns ratio.
        '''
        left_duration = self.left_rhythm.get_duration()
        right_duration = self.right_rhythm.get_duration()
        duration = left_duration / right_duration
        ratio = mathtools.Ratio(duration.pair)
        return ratio

    @property
    def right_markup(self):
        r'''Gets right markup of metric modulation.

        ..  container:: example

            ::

                >>> metric_modulation = indicatortools.MetricModulation(
                ...     left_rhythm=Note("c'4"),
                ...     right_rhythm=Note("c'4."),
                ...     )
                >>> metric_modulation.right_markup

        Returns markup or none.
        '''
        return self._right_markup

    @property
    def right_rhythm(self):
        r'''Gets right tempo of metric modulation.

        ..  container:: example

            ::

                >>> metric_modulation = indicatortools.MetricModulation(
                ...     left_rhythm=Note("c'4"),
                ...     right_rhythm=Note("c'4."),
                ...     )
                >>> metric_modulation.right_rhythm
                Selection([Note("c'4.")])

        Returns selection.
        '''
        return self._right_rhythm



# -*- coding: utf-8 -*-


def to_space_delimited_lowercase(string):
    r'''Changes `string` to space-delimited lowercase.
    
    ..  container:: example

        Changes upper camel case `string` to space-delimited lowercase:

        ::

            >>> stringtools.to_space_delimited_lowercase('LogicalTie')
            'logical tie'

    ..  container:: example

        Changes underscore-delimited `string` to space-delimited lowercase:

        ::

            >>> stringtools.to_space_delimited_lowercase('logical_tie')
            'logical tie'

    ..  container:: example

        Returns space-delimited string unchanged:

        ::

            >>> stringtools.to_space_delimited_lowercase('logical tie')
            'logical tie'

    ..  container:: example

        Returns empty `string` unchanged:

        ::

            >>> stringtools.to_space_delimited_lowercase('')
            ''

    Returns string.
    '''
    if not string:
        return string
    elif string[0].isupper():
        words = []
        current_word = string[0].lower()
        for letter in string[1:]:
            if letter.isupper():
                words.append(current_word)
                current_word = letter.lower()
            else:
                current_word = current_word + letter
        words.append(current_word)
        result = ' '.join(words)
        return result
    return string.replace('_', ' ')



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_leaves_01():
    r'''Leaves constructor can create chords, notes and rests simultaneously.
    '''

    leaves = scoretools.make_leaves([1, (1,2,3), None], [(1, 4)])
    assert isinstance(leaves[0], Note)
    assert isinstance(leaves[1], Chord)
    assert isinstance(leaves[2], Rest)
    for l in leaves:
      assert l.written_duration == Duration(1, 4)


def test_scoretools_make_leaves_02():
    r'''Leaves constructor can create chords, notes and rests
    simultaneously. Contiguous leaves with the same prolation are
    put together inside a tuplet.
    '''

    leaves = scoretools.make_leaves([1, (1, 2, 3), None], [(2, 9), (1, 18), (1,5)])
    assert isinstance(leaves[0], Tuplet)
    assert isinstance(leaves[1], Tuplet)
    tuplet1 = leaves[0]
    assert len(tuplet1) == 2
    assert tuplet1.multiplier == Duration(8, 9)
    assert isinstance(tuplet1[0], Note)
    assert isinstance(tuplet1[1], Chord)
    tuplet2 = leaves[1]
    assert len(tuplet2) == 1
    assert tuplet2.multiplier == Duration(4, 5)
    assert isinstance(tuplet2[0], Rest)

    assert tuplet1[0].written_duration == Duration(2, 8)
    assert tuplet1[1].written_duration == Duration(1, 16)
    assert tuplet2[0].written_duration == Duration(1, 4)


def test_scoretools_make_leaves_03():
    r'''Leaves constructor can createand unprolated chords,
    notes and rests simultaneously.
    '''

    leaves = scoretools.make_leaves([1, (1,2,3), None], [(2, 9), (1,8), (1,5)])
    assert isinstance(leaves[0], Tuplet)
    assert isinstance(leaves[1], Chord)
    assert isinstance(leaves[2], Tuplet)
    tuplet1 = leaves[0]
    assert len(tuplet1) == 1
    assert tuplet1.multiplier == Duration(8, 9)
    assert isinstance(tuplet1[0], Note)
    tuplet2 = leaves[2]
    assert len(tuplet2) == 1
    assert tuplet2.multiplier == Duration(4, 5)
    assert isinstance(tuplet2[0], Rest)


def test_scoretools_make_leaves_04():
    r'''Do not tie rests unless specified.
    '''

    leaves = scoretools.make_leaves([None], [(5, 32), (5, 32)])
    assert len(leaves) == 4
    assert all(len(inspect_(x).get_logical_tie()) == 1 for x in leaves)


def test_scoretools_make_leaves_05():
    r'''Works with quarter-tone pitch numbers.
    '''

    leaves = scoretools.make_leaves([12, 12.5, 13, 13.5], [(1, 4)])
    assert [leaf.written_pitch.numbered_pitch._pitch_number for leaf in leaves] == \
        [12, 12.5, 13, 13.5]


def test_scoretools_make_leaves_06():
    r'''Works with pitch instances.
    '''

    leaves = scoretools.make_leaves([NamedPitch(0)], [(1, 8), (1, 8), (1, 4)])
    assert [leaf.written_pitch.numbered_pitch._pitch_number for leaf in leaves] == [0, 0, 0]


def test_scoretools_make_leaves_07():
    r'''Chords work with pitch-class / octave strings.
    '''

    leaves = scoretools.make_leaves([('C#5', 'Db5')], [Duration(1, 4), Duration(1, 8)])
    staff = Staff(leaves)

    r'''
    \new Staff {
        <cs'' df''>4
        <cs'' df''>8
    }
    '''

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            <cs'' df''>4
            <cs'' df''>8
        }
        '''
        )


def test_scoretools_make_leaves_08():
    r'''Notes work with pitch-class / octave strings.
    '''

    leaves = scoretools.make_leaves(['C#5', 'Db5'], [Duration(1, 4)])
    staff = Staff(leaves)

    r'''
    \new Staff {
        cs''4
        df''4
    }
    '''

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            cs''4
            df''4
        }
        '''
        )


def test_scoretools_make_leaves_09():
    r'''Works with space-delimited string of pitch names.
    '''

    leaves = scoretools.make_leaves("C#5 Db5 c'' fs''", [Duration(1, 4)])
    staff = Staff(leaves)

    r'''
    \new Staff {
        cs''4
        df''4
        c''4
        fs''4
    }
    '''

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            cs''4
            df''4
            c''4
            fs''4
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.quantizationtools.QSchemaItem import QSchemaItem


class BeatwiseQSchemaItem(QSchemaItem):
    '''`BeatwiseQSchemaItem` represents a change of state in the timeline
    of an unmetered quantization process.

    ::

        >>> q_schema_item = quantizationtools.BeatwiseQSchemaItem()
        >>> print(format(q_schema_item))
        quantizationtools.BeatwiseQSchemaItem()

    Define a change in tempo:

    ::

        >>> q_schema_item = quantizationtools.BeatwiseQSchemaItem(
        ...     tempo=((1, 4), 60),
        ...     )
        >>> print(format(q_schema_item))
        quantizationtools.BeatwiseQSchemaItem(
            tempo=indicatortools.Tempo(
                reference_duration=durationtools.Duration(1, 4), 
                units_per_minute=60,
                ),
            )

    Define a change in beatspan:

    ::

        >>> q_schema_item = quantizationtools.BeatwiseQSchemaItem(
        ...     beatspan=(1, 8),
        ...     )
        >>> print(format(q_schema_item))
        quantizationtools.BeatwiseQSchemaItem(
            beatspan=durationtools.Duration(1, 8),
            )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_beatspan',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        beatspan=None,
        search_tree=None,
        tempo=None,
        ):
        QSchemaItem.__init__(
            self,
            search_tree=search_tree,
            tempo=tempo,
            )
        if beatspan is not None:
            beatspan = durationtools.Duration(beatspan)
            assert 0 < beatspan
        self._beatspan = beatspan

    ### PUBLIC PROPERTIES ###

    @property
    def beatspan(self):
        r'''The optionally defined beatspan duration.

        Returns duration or none.
        '''
        return self._beatspan



# -*- coding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools.pitchtools.IntervalClass import IntervalClass


class NamedIntervalClass(IntervalClass):
    '''Named interval-class.

    ..  container:: example

        Initializes descending major second from string:

        ::

            >>> NamedIntervalClass('-M9')
            NamedIntervalClass('-M2')

    ..  container:: example

        Initializes descending major second from quality string and number of
        semitones:

        ::

            >>> NamedIntervalClass(('major', -9))
            NamedIntervalClass('-M2')

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        '_quality_string',
        )

    _acceptable_quality_strings = (
        'perfect',
        'major',
        'minor',
        'diminished',
        'augmented',
        )

    _interval_number_to_interval_string = {
        1: 'unison',
        2: 'second',
        3: 'third',
        4: 'fourth',
        5: 'fifth',
        6: 'sixth',
        7: 'seventh',
        8: 'octave',
        }

    _quality_abbreviation_to_quality_string = {
        'M': 'major',
        'm': 'minor',
        'P': 'perfect',
        'aug': 'augmented',
        'dim': 'diminished',
        }

    _quality_string_to_quality_abbreviation = {
        'major': 'M',
        'minor': 'm',
        'perfect': 'P',
        'augmented': 'aug',
        'diminished': 'dim',
        }

    ### INITIALIZER ###

    def __init__(self, *arguments):
        from abjad.tools import pitchtools
        if len(arguments) == 1 and \
            isinstance(arguments[0], (pitchtools.NamedInterval,
                pitchtools.NamedIntervalClass)):
            quality_string = arguments[0]._quality_string
            number = arguments[0].number
        elif len(arguments) == 1 and isinstance(arguments[0], str):
            match = \
                pitchtools.Interval._interval_name_abbreviation_regex.match(
                arguments[0])
            if match is None:
                message = '{!r} does not have the form of an abbreviation.'
                message = message.format(arguments[0])
                raise ValueError(message)
            direction_string, quality_abbreviation, number_string = \
                match.groups()
            quality_string = \
                NamedIntervalClass._quality_abbreviation_to_quality_string[
                    quality_abbreviation]
            number = int(direction_string + number_string)
        elif len(arguments) == 1 and mathtools.is_pair(arguments[0]):
            quality_string, number = arguments[0]
        elif len(arguments) == 2:
            quality_string, number = arguments
        elif len(arguments) == 0:
            quality_string = 'perfect'
            number = 1
        else:
            message = 'bad input: {!r}.'
            message = message.format(arguments)
            raise TypeError(message)
        if quality_string not in \
            NamedIntervalClass._acceptable_quality_strings:
            message = 'not acceptable quality string: {!r}.'
            message = message.format(quality_string)
            raise ValueError(message)
        if not isinstance(number, int):
            message = 'must be integer: {!r}.'
            message = message.format(number)
            raise TypeError(message)
        if number == 0:
            message = 'must be nonzero: {!r}.'
            message = message.format(number)
            raise ValueError(number)
        sign = mathtools.sign(number)
        abs_number = abs(number)
        if abs_number % 7 == 1 and 8 <= abs_number:
            number = 8
        else:
            number = abs_number % 7
            if number == 0:
                number = 7
        if not number == 1:
            number *= sign
        self._number = number
        self._quality_string = quality_string

    ### SPECIAL METHODS ###

    def __abs__(self):
        r'''Absolute value of named interval-class.

        Returns new named interval-class.
        '''
        return type(self)(abs(self._number))

    def __eq__(self, argument):
        r'''Is true when `argument` is a named interval-class with direction number,
        quality string and number equal to those of this named interval-class.
        Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            if self.direction_number == argument.direction_number:
                if self._quality_string == argument._quality_string:
                    if self.number == argument.number:
                        return True
        return False

    def __float__(self):
        r'''Changes named interval-class to float.

        Returns float.
        '''
        return float(self._number)

    def __hash__(self):
        r'''Hashes named interval-class.

        Returns integer.
        '''
        return hash(repr(self))

    def __int__(self):
        r'''Changes named interval-class to integer.

        Returns integer.
        '''
        return self._number

    def __lt__(self, argument):
        r'''Is true when `argument` is a named interval class with a number greater
        than that of this named interval.
        '''
        from abjad.tools import pitchtools
        if isinstance(argument, type(self)):
            if self.number == argument.number:
                return pitchtools.NamedInterval(self).semitones < \
                    pitchtools.NamedInterval(argument).semitones
            return self.number < argument.number
        return False

    def __ne__(self, argument):
        r'''Is true when named interval-class does not equal `argument`. Otherwise
        false.

        Returns true or false.
        '''
        return not self == argument

    def __str__(self):
        r'''String representation of named interval-class.

        Returns string.
        '''
        return '{}{}{}'.format(
            self.direction_symbol,
            self._quality_abbreviation,
            abs(self.number),
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _format_string(self):
        return '{}{}'.format(
            self.direction_symbol,
            abs(self.number),
            )

    @property
    def _full_name(self):
        strings = []
        if self.direction_string:
            strings.append(self.direction_string)
        strings.extend([self._quality_string, self._interval_string])
        return ' '.join(strings)

    @property
    def _interval_string(self):
        return self._interval_number_to_interval_string[abs(self.number)]

    @property
    def _quality_abbreviation(self):
        return self._quality_string_to_quality_abbreviation[
            self._quality_string]

    ### PUBLIC PROPERTIES ###

    @property
    def direction_number(self):
        r'''Direction number of named interval-class.

        Returns -1, 0 or 1.
        '''
        if self.number < 1:
            return -1
        elif self.number == 1:
            return 0
        else:
            return 1

    @property
    def direction_string(self):
        r'''Direction word of named interval-class.

        Returns string.
        '''
        if self.direction_number == -1:
            return 'descending'
        elif self.direction_number == 0:
            return None
        elif self.direction_number == 1:
            return 'ascending'

    @property
    def direction_symbol(self):
        r'''Direction symbol of named interval-class.

        Returns string.
        '''
        if self.number < 1:
            return '-'
        elif self.number == 1:
            return ''
        else:
            return '+'

    @property
    def quality_string(self):
        r'''Quality string of named interval-class.

        Returns string.
        '''
        return self._quality_string

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch_carriers(class_, pitch_carrier_1, pitch_carrier_2):
        '''Makes named interval-class from `pitch_carrier_1` and
        `pitch_carrier_2`.

        ::

            >>> NamedIntervalClass.from_pitch_carriers(
            ...     NamedPitch(-2),
            ...     NamedPitch(12),
            ...     )
            NamedIntervalClass('+M2')

        Returns named interval-class.
        '''
        from abjad.tools import pitchtools
        named_interval = pitchtools.NamedInterval.from_pitch_carriers(
            pitch_carrier_1, pitch_carrier_2)
        return class_(named_interval)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_move_measure_prolation_to_full_measure_tuplet_01():
    r'''Move 3/12 prolation to full-measure tuplet.
    '''

    measure = Measure((2, 8), [])
    measure.append(r"\times 2/3 { \times 2/3 { c'16 d'16 e'16 } f'8 g'8 }")
    scoretools.move_full_measure_tuplet_prolation_to_measure_time_signature(
        measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/12
            \scaleDurations #'(2 . 3) {
                \times 2/3 {
                    c'16
                    d'16
                    e'16
                }
                f'8
                g'8
            }
        }
        '''
        ), format(measure)

    scoretools.move_measure_prolation_to_full_measure_tuplet(measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 2/8
            \times 2/3 {
                \times 2/3 {
                    c'16
                    d'16
                    e'16
                }
                f'8
                g'8
            }
        }
        '''
        ), format(measure)

    assert inspect_(measure).is_well_formed()


def test_scoretools_move_measure_prolation_to_full_measure_tuplet_02():
    r'''Project time signature without power-of-two denominator
    onto measure with tied note values.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(5, 8), [])
    tuplet.extend("c'8 d'8 e'8 f'8 g'8 a'8")
    measure = Measure((5, 8), [tuplet])
    scoretools.move_full_measure_tuplet_prolation_to_measure_time_signature(
        measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 15/24
            \scaleDurations #'(2 . 3) {
                c'8 ~
                c'32
                d'8 ~
                d'32
                e'8 ~
                e'32
                f'8 ~
                f'32
                g'8 ~
                g'32
                a'8 ~
                a'32
            }
        }
        '''
        ), format(measure)

    scoretools.move_measure_prolation_to_full_measure_tuplet(measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 5/8
            \tweak text #tuplet-number::calc-fraction-text
            \times 5/6 {
                c'8
                d'8
                e'8
                f'8
                g'8
                a'8
            }
        }
        '''
        ), format(measure)

    assert inspect_(measure).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools.topleveltools import attach


def append_spacer_skip_to_underfull_measure(measure):
    r'''Append spacer skip to underfull `measure`:

    ::

        >>> measure = Measure((4, 12), "c'8 d'8 e'8 f'8")
        >>> measure.implicit_scaling = True
        >>> detach(TimeSignature, measure)
        (TimeSignature((4, 12)),)
        >>> new_time_signature = TimeSignature((5, 12))
        >>> attach(new_time_signature, measure)
        >>> measure.is_underfull
        True

    ::

        >>> scoretools.append_spacer_skip_to_underfull_measure(measure)
        Measure((5, 12), "c'8 d'8 e'8 f'8 s1 * 1/8", implicit_scaling=True)

    ..  doctest::

        >>> print(format(measure))
        {
            \time 5/12
            \scaleDurations #'(2 . 3) {
                c'8
                d'8
                e'8
                f'8
                s1 * 1/8
            }
        }

    Append nothing to nonunderfull `measure`.

    Return `measure`.
    '''
    from abjad.tools import scoretools

    assert isinstance(measure, scoretools.Measure)

    if measure.is_underfull:
        target_duration = measure.time_signature.duration
        duration = measure._get_duration()
        skip = scoretools.Skip((1, 1))
        time_signature_multiplier = \
            measure.time_signature.implied_prolation
        new_duration = target_duration - duration
        new_multiplier = new_duration.__div__(time_signature_multiplier)
        attach(new_multiplier, skip)
        measure.append(skip)

    return measure



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import inspect_


class StemTremoloSpanner(Spanner):
    r'''Stem tremolo spanner.

    ::

        >>> staff = Staff("c'32 d'16. e'8 f'4. g'4.")
        >>> tremolo_spanner = spannertools.StemTremoloSpanner()
        >>> attach(tremolo_spanner, staff[:])
        >>> f(staff)
        \new Staff {
            c'32 :256
            d'16. :128
            e'8 :64
            f'4. :32
            g'4. :32
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_minimum_duration',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        minimum_duration=None,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        if minimum_duration is not None:
            minimum_duration = durationtools.Duration(minimum_duration)
        self._minimum_duration = minimum_duration

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new._minimum_duration = self.minimum_duration

    def _get_lilypond_format_bundle(self, leaf):
        from abjad.tools import scoretools
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        prototype = (
            scoretools.Note,
            scoretools.Chord,
            )
        if not isinstance(leaf, prototype):
            return lilypond_format_bundle
        logical_tie = inspect_(leaf).get_logical_tie()
        if (self.minimum_duration is not None and
            logical_tie.get_duration() < self.minimum_duration):
            return lilypond_format_bundle
        flag_count = leaf.written_duration.flag_count
        tremolo_count = 32
        if flag_count:
            tremolo_count *= pow(2, flag_count)
        tremolo_string = ':{}'.format(tremolo_count)
        lilypond_format_bundle.right.stem_tremolos.append(tremolo_string)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def minimum_duration(self):
        r'''Gets minimum logical tie duration to which stem tremolos will be
        applied.

        Returns minimum duration.
        '''
        return self._minimum_duration



# -*- coding: utf-8 -*-
import abjad


class ScoreTemplate(abjad.abctools.AbjadObject):
    r'''Score template.
    '''

    def __call__(self):
        r'''Calls score template.

        Returns score.
        '''
        voice = abjad.Voice(name='Example Voice')
        staff = abjad.Staff([voice], name='Example Staff')
        score = abjad.Score([staff], name='Example Score')
        return score



# -*- coding: utf-8 -*-
from abjad.tools import schemetools


def configure_lilypond_file(lilypond_file):
    r'''Configures LilyPond file.
    '''

    lilypond_file._default_paper_size = '11x17', 'portrait'
    lilypond_file._global_staff_size = 12
    lilypond_file.layout_block.indent = 0
    lilypond_file.layout_block.ragged_right = True
    lilypond_file.paper_block.ragged_bottom = True
    spacing_vector = schemetools.make_spacing_vector(0, 0, 8, 0)
    lilypond_file.paper_block.system_system_spacing = spacing_vector


# -*- coding: utf-8 -*-
import re
from abjad.tools import pitchtools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadObject


class ScaleDegree(AbjadObject):
    '''A diatonic scale degree such as 1, 2, 3, 4, 5, 6, 7 and
    also chromatic alterations including flat-2, flat-3, flat-6, etc.

    ::

        >>> scale_degree = tonalanalysistools.ScaleDegree('#4')
        >>> scale_degree
        ScaleDegree('sharp', 4)

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_accidental',
        '_number',
        )

    _numeral_to_number_name = {
        1: 'one',
        2: 'two',
        3: 'three',
        4: 'four',
        5: 'five',
        6: 'six',
        7: 'seven',
        8: 'eight',
        9: 'nine',
        10: 'ten',
        11: 'eleven',
        12: 'twelve',
        13: 'thirteen',
        14: 'fourteen',
        15: 'fifteen',
        }

    _roman_numeral_string_to_scale_degree_number = {
        'I': 1,
        'II': 2,
        'III': 3,
        'IV': 4,
        'V': 5,
        'VI': 6,
        'VII': 7,
        }

    _scale_degree_number_to_roman_numeral_string = {
        1: 'I',
        2: 'II',
        3: 'III',
        4: 'IV',
        5: 'V',
        6: 'VI',
        7: 'VII',
        }

    _scale_degree_number_to_scale_degree_name = {
        1: 'tonic',
        2: 'superdominant',
        3: 'mediant',
        4: 'subdominant',
        5: 'dominant',
        6: 'submediant',
        7: 'leading tone',
        }

    _symbolic_string_regex = re.compile(r'([#|b]*)([i|I|v|V|\d]+)')

    ### INITIALIZER ###

    def __init__(self, *arguments):
        if len(arguments) == 0:
            accidental, number = self._initialize_by_number(1)
        elif len(arguments) == 1 and isinstance(arguments[0], type(self)):
            accidental, number = self._initialize_by_scale_degree(*arguments)
        elif len(arguments) == 1 and arguments[0] in self._acceptable_numbers:
            accidental, number = self._initialize_by_number(*arguments)
        elif len(arguments) == 1 and isinstance(arguments[0], tuple):
            accidental, number = self._initialize_by_pair(*arguments)
        elif len(arguments) == 1 and isinstance(arguments[0], str):
            accidental, number = self._initialize_by_symbolic_string(*arguments)
        elif len(arguments) == 2 and arguments[1] in self._acceptable_numbers:
            accidental, number = \
                self._initialize_by_accidental_and_number(*arguments)
        else:
            arg_string = ', '.join([str(x) for x in arguments])
            message = 'can not initialize scale degree: {}.'
            raise ValueError(message.format(arg_string))
        self._accidental = accidental
        self._number = number

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a scale degree with number and accidental equal
        to those of this scale degree.

        ::

            >>> scale_degree == tonalanalysistools.ScaleDegree('#4')
            True

        Otherwise false:

        ::

            >>> scale_degree == tonalanalysistools.ScaleDegree(4)
            False

        Returns true or false.
        '''
        if isinstance(arg, type(self)):
            if self.number == arg.number:
                if self.accidental == arg.accidental:
                    return True
        return False

    def __hash__(self):
        r'''Hashes scale degree.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(ScaleDegree, self).__hash__()

    def __ne__(self, arg):
        r'''Is true when `arg` does not equal scale degree. Otherwise false.

        Returns true or false.
        '''
        return not self == arg

    def __repr__(self):
        r'''Gets interpreter representation of scale degree.

        ::

            >>> scale_degree
            ScaleDegree('sharp', 4)

        Returns string.
        '''
        return systemtools.StorageFormatAgent(self).get_repr_format()

    def __str__(self):
        r'''String representation of scale degree.

        ::

            >>> str(scale_degree)
            '#4'

        Returns string.
        '''
        return '{}{}'.format(
            self.accidental.symbolic_string, self.number)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = [self.number]
        if self.accidental.is_adjusted:
            values = [self.accidental.name, self.number]
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=values,
            )

    def _initialize_by_accidental_and_number(self, accidental, number):
        accidental = pitchtools.Accidental(accidental)
        return accidental, number

    def _initialize_by_number(self, number):
        accidental = pitchtools.Accidental(None)
        return accidental, number

    def _initialize_by_pair(self, pair):
        accidental, number = pair
        return self._initialize_by_accidental_and_number(accidental, number)

    def _initialize_by_scale_degree(self, scale_degree):
        accidental = scale_degree.accidental
        number = scale_degree.number
        return self._initialize_by_accidental_and_number(accidental, number)

    def _initialize_by_symbolic_string(self, symbolic_string):
        groups = self._symbolic_string_regex.match(symbolic_string).groups()
        accidental, roman_numeral = groups
        accidental = pitchtools.Accidental(accidental)
        roman_numeral = roman_numeral.upper()
        try:
            number = self._roman_numeral_string_to_scale_degree_number[
                roman_numeral]
        except KeyError:
            number = int(roman_numeral)
        return accidental, number

    ### PUBLIC METHODS ###

    def apply_accidental(self, accidental):
        r'''Applies accidental to scale degree.

        ::

            >>> scale_degree.apply_accidental('ff')
            ScaleDegree('flat', 4)

        Returns new scale degree.
        '''
        accidental = pitchtools.Accidental(accidental)
        new_accidental = self.accidental + accidental
        return type(self)(new_accidental, self.number)

    ### PRIVATE PROPERTIES ###

    @property
    def _acceptable_numbers(self):
        return tuple(range(1, 16))

    ### PUBLIC PROPERTIES ###

    @property
    def accidental(self):
        r'''Accidental of scale degree.

        ::

            >>> scale_degree.accidental
            Accidental('s')

        Returns accidental.
        '''
        return self._accidental

    @property
    def name(self):
        r'''Name of scale degree.

        ::

            >>> tonalanalysistools.ScaleDegree(4).name
            'subdominant'

        Returns string.
        '''
        if not self.accidental.is_adjusted:
            return self._scale_degree_number_to_scale_degree_name[self.number]
        else:
            raise NotImplementedError

    @property
    def number(self):
        r'''Number of scale degree.

        ::

            >>> scale_degree.number
            4

        Returns integer from 1 to 7, inclusive.
        '''
        return self._number

    @property
    def roman_numeral_string(self):
        r'''Roman numeral string of scale degree.

        ::

            >>> scale_degree.roman_numeral_string
            'IV'

        Returns string.
        '''
        string = self._scale_degree_number_to_roman_numeral_string[self.number]
        return string

    @property
    def symbolic_string(self):
        r'''Symbolic string of scale degree.

        ::

            >>> scale_degree.symbolic_string
            '#IV'

        Returns string.
        '''
        return '{}{}'.format(self.accidental.symbolic_string,
            self.roman_numeral_string)

    @property
    def title_string(self):
        r'''Title string of scale degree.

        ::

            >>> scale_degree.title_string
            'SharpFour'

        Returns string.
        '''
        if not self.accidental.name == 'natural':
            accidental = self.accidental.name
        else:
            accidental = ''
        number = self._numeral_to_number_name[self.number]
        return '{}{}'.format(accidental.title(), number.title())



# -*- coding: utf-8 -*-
import copy
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import mathtools
from abjad.tools import systemtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import set_
from abjad.tools.scoretools.FixedDurationContainer \
    import FixedDurationContainer


class Measure(FixedDurationContainer):
    r'''A measure.

    ..  container:: example

        >>> measure = Measure((4, 8), "c'8 d'8 e'8 f'8")
        >>> show(measure) # doctest: +SKIP

    ..  doctest::

        >>> print(format(measure))
        {
            \time 4/8
            c'8
            d'8
            e'8
            f'8
        }

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Containers'

    __slots__ = (
        '_always_format_time_signature',
        '_automatically_adjust_time_signature',
        '_measure_number',
        '_implicit_scaling',
        )

    _is_counttime_component = True

    ### INITIALIZER ###

    def __init__(
        self,
        time_signature=None,
        music=None,
        implicit_scaling=False,
        ):
        # set time signature adjustment before contents initialization
        self._automatically_adjust_time_signature = False
        time_signature = time_signature or (4, 4)
        self.implicit_scaling = bool(implicit_scaling)
        FixedDurationContainer.__init__(self, time_signature, music)
        self._always_format_time_signature = False
        self._measure_number = None
        time_signature = indicatortools.TimeSignature(time_signature)
        attach(time_signature, self)

    ### SPECIAL METHODS ###

    def __delitem__(self, i):
        r'''Deletes measure item `i`.

        ..  container:: example

            ::

                >>> measure = Measure((4, 8), "c'8 d'8 e'8 f'8")
                >>> measure.automatically_adjust_time_signature = True
                >>> show(measure) # doctest: +SKIP

            ::

                >>> del(measure[1])
                >>> show(measure) # doctest: +SKIP

            ..  doctest::

                {
                    \time 3/8
                    c'8
                    e'8
                    f'8
                }

        Returns none.
        '''
        old_time_signature = self.time_signature
        old_denominator = getattr(old_time_signature, 'denominator', None)
        FixedDurationContainer.__delitem__(self, i)
        self._conditionally_adjust_time_signature(old_denominator)

    def __getnewargs__(self):
        r'''Gets new arguments of measure.

        Returns pair.
        '''
        time_signature = self.time_signature
        return (time_signature.pair,)

    def __repr__(self):
        r'''Gets interpreter representation of measure.

        ..  container:: example

            ::

                >>> measure = Measure((3, 8), "c'8 d'8 e'8")
                >>> show(measure) # doctest: +SKIP

            ::

                >>> measure
                Measure((3, 8), "c'8 d'8 e'8")

        Returns string.
        '''
        class_name = type(self).__name__
        indicator = self._get_indicator(indicatortools.TimeSignature)
        forced_time_signature = indicator
        forced_time_signature = forced_time_signature.pair
        summary = self._contents_summary
        if forced_time_signature and len(self):
            if self.implicit_scaling:
                result = '{}({!s}, {!r}, implicit_scaling=True)'
                result = result.format(
                    class_name,
                    forced_time_signature,
                    summary,
                    )
            else:
                result = '{}({!s}, {!r})'
                result = result.format(
                    class_name,
                    forced_time_signature,
                    summary,
                    )
            return result
        elif forced_time_signature:
            if self.implicit_scaling:
                string = '{}({!s}, implicit_scaling=True)'
            else:
                string = '{}({!s})'
            string = string.format(class_name, forced_time_signature)
            return string
        else:
            if self.implicit_scaling:
                return '{}(implicit_scaling=True)'.format(class_name)
            else:
                return '{}()'.format(class_name)

    def __setitem__(self, i, argument):
        r'''Sets measure item `i` to `argument`.

        ..  container:: example

            ::

                >>> measure = Measure((4, 8), "c'8 d'8 e'8 f'8")
                >>> measure.automatically_adjust_time_signature = True
                >>> show(measure) # doctest: +SKIP

            ::

                >>> measure[1] = Note("ds'8.")
                >>> show(measure) # doctest: +SKIP

            ..  doctest::

                {
                    \time 9/16
                    c'8
                    ds'8.
                    e'8
                    f'8
                }

        Returns none.
        '''
        old_time_signature = self.time_signature
        old_denominator = getattr(old_time_signature, 'denominator', None)
        FixedDurationContainer.__setitem__(self, i, argument)
        self._conditionally_adjust_time_signature(old_denominator)

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_representation(self):
        if not self:
            return '| {!s} |'.format(self.time_signature)
        return '| {!s} {} |'.format(
            self.time_signature,
            self._contents_summary,
            )

    @property
    def _preprolated_duration(self):
        time_signature_prolation = 1
        if self.implicit_scaling:
            time_signature_prolation = self.time_signature.implied_prolation
        return time_signature_prolation * self._contents_duration

    ### PRIVATE METHODS ###

    def _all_contents_are_scalable_by_multiplier(self, multiplier):
        from abjad.tools import scoretools
        multiplier = durationtools.Multiplier(multiplier)
        for component in self:
            if isinstance(component, scoretools.Leaf):
                candidate_duration = multiplier * component.written_duration
                if not candidate_duration.is_assignable:
                    return False
        return True

    def _as_graphviz_node(self):
        from abjad.tools import documentationtools
        from abjad.tools import scoretools
        node = scoretools.Component._as_graphviz_node(self)
        fraction = mathtools.NonreducedFraction(
            self.time_signature.numerator,
            self.time_signature.denominator,
            )
        node[0].extend([
            graphtools.GraphvizTableRow([
                graphtools.GraphvizTableCell(
                    label=type(self).__name__,
                    attributes={'border': 0},
                    ),
                ]),
            graphtools.GraphvizTableHorizontalRule(),
            graphtools.GraphvizTableRow([
                graphtools.GraphvizTableCell(
                    label=str(fraction),
                    attributes={'border': 0},
                    ),
                ]),
            ])
        return node

    def _check_duration(self):
        effective_time_signature = self.time_signature
        if effective_time_signature.has_non_power_of_two_denominator and \
            effective_time_signature.suppress:
            message = 'can not suppress time signature'
            message += ' with non-power-of-two denominator.'
            raise Exception(message)
        if effective_time_signature.duration < self._preprolated_duration:
            raise OverfullContainerError
        if self._preprolated_duration < effective_time_signature.duration:
            raise UnderfullContainerError

    def _conditionally_adjust_time_signature(self, old_denominator):
        if self.automatically_adjust_time_signature:
            naive_time_signature = self._preprolated_duration
            better_time_signature = \
                mathtools.NonreducedFraction(naive_time_signature)
            better_time_signature = \
                better_time_signature.with_denominator(old_denominator)
            better_time_signature = \
                indicatortools.TimeSignature(better_time_signature)
            detach(indicatortools.TimeSignature, self)
            attach(better_time_signature, self)

    # essentially the same as container version of method;
    # the definition given here adds one line to remove
    # time signature immediately after instantiation
    # because the indicator-copying code will then provide time signature.
    def _copy_with_indicators_but_without_children_or_spanners(self):
        from abjad.tools import indicatortools
        new = type(self)(*self.__getnewargs__())
        # only the following line differs from Container
        detach(indicatortools.TimeSignature, new)
        if getattr(self, '_lilypond_grob_name_manager', None) is not None:
            new._lilypond_grob_name_manager = copy.copy(override(self))
        if getattr(self, '_lilypond_setting_name_manager', None) is not None:
            new._lilypond_setting_name_manager = copy.copy(set_(self))
        for indicator in self._get_indicators():
            new_indicator = copy.copy(indicator)
            attach(new_indicator, new)
        new.is_simultaneous = self.is_simultaneous
        new.implicit_scaling = self.implicit_scaling
        return new

    @staticmethod
    def _duration_and_possible_denominators_to_time_signature(
        duration,
        denominators=None,
        factor=None,
        ):
        # check input
        duration = durationtools.Duration(duration)
        if denominators is not None:
            if factor is not None:
                denominators = [
                    d for d in denominators
                    if factor in mathtools.factors(d)
                    ]
            for desired_denominator in sorted(denominators):
                nonreduced_fraction = mathtools.NonreducedFraction(duration)
                candidate_pair = \
                    nonreduced_fraction.with_denominator(desired_denominator)
                if candidate_pair.denominator == desired_denominator:
                    return indicatortools.TimeSignature(candidate_pair)
        if factor is not None:
            if factor in mathtools.factors(duration.denominator):
                return indicatortools.TimeSignature(duration)
            else:
                time_signature_numerator = factor * duration.numerator
                time_signature_denominator = factor * duration.denominator
                return indicatortools.TimeSignature(
                    (time_signature_numerator, time_signature_denominator))
        else:
            return indicatortools.TimeSignature(duration)

    def _format_content_pieces(self):
        from abjad.tools import systemtools
        result = []
        if self.has_non_power_of_two_denominator and \
            type(self) is Measure and \
            self.implicit_scaling:
            indent = systemtools.LilyPondFormatManager.indent
            string = "{}\\scaleDurations #'({} . {}) {{"
            string = string.format(
                indent,
                self.implied_prolation.numerator,
                self.implied_prolation.denominator,
                )
            result.append(string)
            pieces = FixedDurationContainer._format_content_pieces(self)
            pieces = [indent + x for x in pieces]
            result.extend(pieces)
            result.append(indent + '}')
        else:
            result.extend(FixedDurationContainer._format_content_pieces(self))
        return result

    def _format_opening_slot(self, bundle):
        result = []
        result.append(('comments', bundle.opening.comments))
        result.append(('grob overrides', bundle.grob_overrides))
        result.append(('context settings', bundle.context_settings))
        result.append(('indicators', bundle.opening.indicators))
        return self._format_slot_contributions_with_indent(result)

    def _get_format_specification(self):
        names = []
        if self.implicit_scaling:
            names.append('implicit_scaling')
        return systemtools.FormatSpecification(
            client=self,
            repr_args_values=[
                self.time_signature.pair,
                self._contents_summary,
                ],
            storage_format_args_values=[
                self.time_signature,
                self._contents_summary,
                ],
            storage_format_kwargs_names=names,
            )

    @staticmethod
    def _get_likely_multiplier_of_components(components):
        pass
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        from abjad.tools import sequencetools
        assert all(isinstance(x, scoretools.Component) for x in components)
        logical_tie_duration_numerators = []
        items = iterate(components).by_topmost_logical_ties_and_components()
        for item in items:
            if isinstance(item, selectiontools.LogicalTie):
                logical_tie_duration = item._preprolated_duration
                numerator = logical_tie_duration.numerator
                logical_tie_duration_numerators.append(numerator)
        numerators = sequencetools.Sequence(logical_tie_duration_numerators)
        if len(numerators.remove_repeats()) == 1:
            numerator = logical_tie_duration_numerators[0]
            denominator = mathtools.greatest_power_of_two_less_equal(numerator)
            pair = (numerator, denominator)
            likely_multiplier = durationtools.Multiplier(*pair)
            return likely_multiplier

    def _get_lilypond_format(self):
        self._check_duration()
        return self._format_component()

    # TODO: see if self._scale can be combined with
    #       with self.scale_and_adjust_time_signature()
    def _scale(self, multiplier=None):
        from abjad.tools import indicatortools
        if multiplier is None:
            return
        multiplier = durationtools.Multiplier(multiplier)
        old_time_signature = self.time_signature
        if (mathtools.is_nonnegative_integer_power_of_two(multiplier) and
            1 <= multiplier):
            old_numerator = old_time_signature.numerator
            old_denominator = old_time_signature.denominator
            new_denominator = old_denominator // multiplier.numerator
            pair = (old_numerator, new_denominator)
            new_time_signature = indicatortools.TimeSignature(pair)
        else:
            old_denominator = old_time_signature.denominator
            old_duration = old_time_signature.duration
            new_duration = multiplier * old_duration
            new_time_signature = \
                self._duration_and_possible_denominators_to_time_signature(
                    new_duration,
                    [old_denominator],
                    multiplier.denominator,
                    )
        detach(indicatortools.TimeSignature, self)
        attach(new_time_signature, self)
        contents_multiplier_denominator = \
            mathtools.greatest_power_of_two_less_equal(multiplier.denominator)
        pair = (multiplier.numerator, contents_multiplier_denominator)
        contents_multiplier = durationtools.Multiplier(*pair)
        self._scale_contents(contents_multiplier)

    ### PUBLIC PROPERTIES ###

    @property
    def always_format_time_signature(self):
        '''Gets and sets flag to indicate whether time signature
        should appear in LilyPond format even when not expected.

        ..  container:: example

            ::

                >>> measure.always_format_time_signature
                False

        Set to true when necessary to print the same signature repeatedly.

        Defaults to false.

        Returns true or false.
        '''
        return self._always_format_time_signature

    @always_format_time_signature.setter
    def always_format_time_signature(self, argument):
        assert isinstance(argument, bool)
        self._always_format_time_signature = argument

    @property
    def automatically_adjust_time_signature(self):
        '''Gets and sets flag to indicate whether time signature
        should update automatically following mutation.

        ..  container:: example

            ::

                >>> measure = Measure((3, 4), "c' d' e'")
                >>> show(measure) # doctest: +SKIP

            ::

                >>> measure.automatically_adjust_time_signature = True
                >>> measure.append('r')
                >>> show(measure) # doctest: +SKIP

        Defaults to false.

        Returns true or false.
        '''
        return self._automatically_adjust_time_signature

    @automatically_adjust_time_signature.setter
    def automatically_adjust_time_signature(self, argument):
        assert isinstance(argument, bool)
        self._automatically_adjust_time_signature = argument

    @property
    def has_non_power_of_two_denominator(self):
        r'''Is true when measure time signature denominator
        is not an integer power of 2.

        ..  container:: example

            ::

                >>> measure = Measure((5, 9), "c'8 d' e' f' g'")
                >>> measure.implicit_scaling = True
                >>> show(measure) # doctest: +SKIP

            ::

                >>> measure.has_non_power_of_two_denominator
                True

        Otherwise false:

        ..  container:: example

            ::

                >>> measure = Measure((5, 8), "c'8 d' e' f' g'")
                >>> show(measure) # doctest: +SKIP

            ::

                >>> measure.has_non_power_of_two_denominator
                False

        Returns true or false.
        '''
        time_signature = self.time_signature
        return time_signature.has_non_power_of_two_denominator

    @property
    def has_power_of_two_denominator(self):
        r'''Is true when measure time signature denominator
        is an integer power of 2.

        ..  container:: example

            ::

                >>> measure = Measure((5, 8), "c'8 d' e' f' g'")
                >>> show(measure) # doctest: +SKIP

            ::

                >>> measure.has_power_of_two_denominator
                True

        Otherwise false:

        ..  container:: example

            ::

                >>> measure = Measure((5, 9), "c'8 d' e' f' g'")
                >>> measure.implicit_scaling = True
                >>> show(measure) # doctest: +SKIP

            ::

                >>> measure.has_power_of_two_denominator
                False

        Returns true or false.
        '''
        return not self.has_non_power_of_two_denominator

    @property
    def implicit_scaling(self):
        r'''Is true when measure should scale contents. Otherwise false.

        Returns true or false.
        '''
        return self._implicit_scaling

    @implicit_scaling.setter
    def implicit_scaling(self, arg):
        assert isinstance(arg, bool)
        self._implicit_scaling = arg

    @property
    def implied_prolation(self):
        r'''Implied prolation of measure.

        ..  container:: example

            Measures with implicit scaling scale the duration of their
            contents:

            ::

                >>> measure = Measure((5, 12), "c'8 d'8 e'8 f'8 g'8")
                >>> measure.implicit_scaling = True
                >>> show(measure) # doctest: +SKIP

            ::

                >>> measure.implied_prolation
                Multiplier(2, 3)

            ::

                >>> for note in measure:
                ...     note, inspect_(note).get_duration()
                (Note("c'8"), Duration(1, 12))
                (Note("d'8"), Duration(1, 12))
                (Note("e'8"), Duration(1, 12))
                (Note("f'8"), Duration(1, 12))
                (Note("g'8"), Duration(1, 12))

        ..  container:: example

            Measures without implicit scaling turned on do not
            scale the duration of their contents:

            ::

                >>> measure = Measure((5, 12), [])
                >>> measure.implicit_scaling = False

            ::

                >>> measure.implied_prolation
                Multiplier(1, 1)

        Returns positive multiplier.
        '''
        if self.implicit_scaling:
            time_signature = self.time_signature
            return time_signature.implied_prolation
        return durationtools.Multiplier(1)

    @property
    def is_full(self):
        r'''Is true when measure duration equals time signature duration.

        ..  container:: example

            ::

                >>> measure = Measure((4, 8), "c'8 d'8 e'8 f'8")
                >>> show(measure) # doctest: +SKIP

            ::

                >>> measure.is_full
                True

        Otherwise false.

        Returns true or false.
        '''
        return FixedDurationContainer.is_full.fget(self)

    @property
    def is_misfilled(self):
        '''Is true when measure is either underfull or overfull.

        ..  container:: example

            ::

                >>> measure = Measure((3, 4), "c'4 d'4 e'4 f'4")
                >>> measure.is_misfilled
                True

        Otherwise false:

        ..  container:: example

            ::

                >>> measure = Measure((3, 4), "c' d' e'")
                >>> show(measure) # doctest: +SKIP

            ::

                >>> measure.is_misfilled
                False

        Returns true or false.
        '''
        return FixedDurationContainer.is_overfull.fget(self)

    @property
    def is_overfull(self):
        '''Is true when measure duration is greater than time signature
        duration.

        ..  container:: example

            ::

                >>> measure = Measure((3, 4), "c'4 d' e' f'")

            ::

                >>> measure.is_overfull
                True

        Otherwise false.

        Returns true or false.
        '''
        return FixedDurationContainer.is_overfull.fget(self)

    @property
    def is_underfull(self):
        '''Is true when measure duration is less than time signature duration.

        ..  container:: example

            ::

                >>> measure = Measure((3, 4), "c'4 d'")

            ::

                >>> measure.is_underfull
                True

        Otherwise false.

        Returns true or false.
        '''
        return FixedDurationContainer.is_underfull.fget(self)

    @property
    def measure_number(self):
        r'''Gets 1-indexed measure number.

        ..  container:: example

            ::

                >>> staff = Staff()
                >>> staff.append(Measure((3, 4), "c' d' e'"))
                >>> staff.append(Measure((2, 4), "f' g'"))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(staff))
                \new Staff {
                    {
                        \time 3/4
                        c'4
                        d'4
                        e'4
                    }
                    {
                        \time 2/4
                        f'4
                        g'4
                    }
                }

            ::

                >>> staff[0].measure_number
                1

            ::

                >>> staff[1].measure_number
                2

        Returns positive integer.
        '''
        self._update_now(offsets=True)
        return self._measure_number

    @property
    def target_duration(self):
        r'''Gets target duration of measure.

        Target duration of measure is always equal to duration
        of effective time signature.

        ..  container:: example

            ::

                >>> measure = Measure((3, 4), "c'4 d'4 e'4")
                >>> measure.target_duration
                Duration(3, 4)

        Returns duration.
        '''
        return self.time_signature.duration

    @property
    def time_signature(self):
        r'''Gets effective time signature of measure.

        ..  container:: example

            ::

                >>> measure.time_signature
                TimeSignature((3, 4))

        Returns time signature or none.
        '''
        from abjad.tools import indicatortools
        return self._get_effective(indicatortools.TimeSignature)

    ### PUBLIC METHODS ###

    @classmethod
    def from_selections(class_, selections, time_signatures=None):
        r'''Makes a selection of measures from `selections`.

        Returns selections.
        '''
        from abjad.tools import scoretools
        assert len(selections)
        if not time_signatures:
            time_signatures = [_.get_duration() for _ in selections]
        assert len(selections) == len(time_signatures)
        assert [_.get_duration() for _ in selections] == \
            [durationtools.Duration(_) for _ in time_signatures]
        measures = scoretools.make_spacer_skip_measures(time_signatures)
        temporary_voice = scoretools.Voice(measures)
        mutate(temporary_voice).replace_measure_contents(selections)
        temporary_voice[:] = []
        return measures

    # TODO: see if self._scale can be combined with
    #       with self.scale_and_adjust_time_signature()
    def scale_and_adjust_time_signature(self, multiplier=None):
        r'''Scales `measure` by `multiplier` and adjusts time signature.

        ..  container:: example

            Scales measure by non-power-of-two multiplier:

            ::

                >>> measure = Measure((3, 8), "c'8 d'8 e'8")
                >>> measure.implicit_scaling = True
                >>> show(measure) # doctest: +SKIP

            ..  doctest::

                >>> print(format(measure))
                {
                    \time 3/8
                    c'8
                    d'8
                    e'8
                }

            ::

                >>> measure.scale_and_adjust_time_signature(Multiplier(2, 3))
                >>> show(measure) # doctest: +SKIP

            ..  doctest::

                >>> print(format(measure))
                {
                    \time 3/12
                    \scaleDurations #'(2 . 3) {
                        c'8
                        d'8
                        e'8
                    }
                }

        Returns none.
        '''
        from abjad.tools import indicatortools
        if multiplier == 0:
            raise ZeroDivisionError
        old_time_signature = self.time_signature
        old_pair = \
            (old_time_signature.numerator, old_time_signature.denominator)
        old_multiplier = old_time_signature.implied_prolation
        old_multiplier_pair = \
            (old_multiplier.numerator, old_multiplier.denominator)
        multiplied_pair = mathtools.NonreducedFraction(old_multiplier_pair)
        multiplied_pair = multiplied_pair.multiply_without_reducing(multiplier)
        multiplied_pair = multiplied_pair.pair
        reduced_pair = mathtools.NonreducedFraction(old_multiplier_pair)
        reduced_pair = reduced_pair.multiply_with_cross_cancelation(multiplier)
        reduced_pair = reduced_pair.pair
        if reduced_pair != multiplied_pair:
            new_pair = mathtools.NonreducedFraction(old_pair)
            new_pair = \
                new_pair.multiply_with_numerator_preservation(multiplier)
            new_time_signature = indicatortools.TimeSignature(new_pair)
            detach(indicatortools.TimeSignature, self)
            attach(new_time_signature, self)
            remaining_multiplier = durationtools.Multiplier(reduced_pair)
            if remaining_multiplier != durationtools.Multiplier(1):
                self._scale_contents(remaining_multiplier)
        elif self._all_contents_are_scalable_by_multiplier(multiplier):
            self._scale_contents(multiplier)
            if (
                old_time_signature.has_non_power_of_two_denominator or
                not mathtools.is_nonnegative_integer_power_of_two(multiplier)
                ):
                new_pair = mathtools.NonreducedFraction(old_pair)
                new_pair = new_pair.multiply_with_cross_cancelation(multiplier)
                new_pair = new_pair.pair
            # multiplier is a negative power of two, like 1/2, 1/4, etc.
            elif multiplier < durationtools.Multiplier(0):
                new_pair = \
                    mathtools.NonreducedFraction.multiply_without_reducing(
                        old_pair, multiplier)
            # multiplier is a nonnegative power of two, like 0, 1, 2, 4, etc.
            elif durationtools.Multiplier(0) < multiplier:
                new_pair = mathtools.NonreducedFraction(old_pair)
                new_pair = new_pair.multiply_with_numerator_preservation(
                    multiplier)
            elif multiplier == durationtools.Multiplier(0):
                raise ZeroDivisionError
            new_time_signature = indicatortools.TimeSignature(new_pair)
            detach(indicatortools.TimeSignature, self)
            attach(new_time_signature, self)
            if new_time_signature.has_non_power_of_two_denominator:
                self.implicit_scaling = True
        else:
            new_pair = mathtools.NonreducedFraction(old_pair)
            new_pair = new_pair.multiply_with_numerator_preservation(
                multiplier)
            new_time_signature = indicatortools.TimeSignature(new_pair)
            detach(indicatortools.TimeSignature, self)
            attach(new_time_signature, self)
            if new_time_signature.has_non_power_of_two_denominator:
                self.implicit_scaling = True
            remaining_multiplier = \
                multiplier / new_time_signature.implied_prolation
            if remaining_multiplier != durationtools.Multiplier(1):
                self._scale_contents(remaining_multiplier)



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_lilypondproxytools_LilyPondGrobNameManager___eq___01():

    note_1 = Note("c'4")
    override(note_1).note_head.color = 'red'
    override(note_1).stem.color = 'red'

    note_2 = Note("c'4")
    override(note_2).note_head.color = 'red'
    override(note_2).stem.color = 'red'

    note_3 = Note("c'4")
    override(note_3).note_head.color = 'red'

    grob_override_component_plug_in_1 = override(note_1)
    grob_override_component_plug_in_2 = override(note_2)
    grob_override_component_plug_in_3 = override(note_3)

    assert      grob_override_component_plug_in_1 == grob_override_component_plug_in_1
    assert      grob_override_component_plug_in_1 == grob_override_component_plug_in_2
    assert not grob_override_component_plug_in_1 == grob_override_component_plug_in_3
    assert      grob_override_component_plug_in_2 == grob_override_component_plug_in_1
    assert      grob_override_component_plug_in_2 == grob_override_component_plug_in_2
    assert not grob_override_component_plug_in_2 == grob_override_component_plug_in_3
    assert not grob_override_component_plug_in_3 == grob_override_component_plug_in_1
    assert not grob_override_component_plug_in_3 == grob_override_component_plug_in_2
    assert      grob_override_component_plug_in_3 == grob_override_component_plug_in_3



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Component__logical_measure_number_01():

    staff = Staff("c'4 d' e' f' g' a' b' c''")
    attach(TimeSignature((2, 4)), staff[0])
    attach(TimeSignature((3, 4)), staff[4])
    attach(TimeSignature((1, 4)), staff[7])
    staff._update_logical_measure_numbers()

    assert staff._logical_measure_number == 1
    assert staff[0]._logical_measure_number == 1
    assert staff[1]._logical_measure_number == 1
    assert staff[2]._logical_measure_number == 2
    assert staff[3]._logical_measure_number == 2
    assert staff[4]._logical_measure_number == 3
    assert staff[5]._logical_measure_number == 3
    assert staff[6]._logical_measure_number == 3
    assert staff[7]._logical_measure_number == 4


def test_scoretools_Component__logical_measure_number_02():

    
    staff = Staff()
    staff.append(Measure((2, 4), "c'4 d'4"))
    staff.append(Measure((2, 4), "e'4 f'4"))
    staff.append(Measure((3, 4), "g'4 a'4 b'4"))
    staff.append(Measure((1, 4), "c''4"))
    staff._update_logical_measure_numbers()

    assert staff._logical_measure_number == 1

    assert staff[0]._logical_measure_number == 1
    assert staff[1]._logical_measure_number == 2
    assert staff[2]._logical_measure_number == 3
    assert staff[3]._logical_measure_number == 4

    leaves = iterate(staff).by_leaf()
    leaves = list(leaves)
    assert leaves[0]._logical_measure_number == 1
    assert leaves[1]._logical_measure_number == 1
    assert leaves[2]._logical_measure_number == 2
    assert leaves[3]._logical_measure_number == 2
    assert leaves[4]._logical_measure_number == 3
    assert leaves[5]._logical_measure_number == 3
    assert leaves[6]._logical_measure_number == 3
    assert leaves[7]._logical_measure_number == 4


def test_scoretools_Component__logical_measure_number_03():
    r'''Works with implicit time signatures.
    '''

    staff = Staff("c'4 d' e' f' g' a' b' c''")
    staff._update_logical_measure_numbers()

    assert staff._logical_measure_number == 1
    assert staff[0]._logical_measure_number == 1
    assert staff[1]._logical_measure_number == 1
    assert staff[2]._logical_measure_number == 1
    assert staff[3]._logical_measure_number == 1
    assert staff[4]._logical_measure_number == 2
    assert staff[5]._logical_measure_number == 2
    assert staff[6]._logical_measure_number == 2
    assert staff[7]._logical_measure_number == 2


def test_scoretools_Component__logical_measure_number_04():
    r'''Works with a mix of implicit and explicit time signatures.
    '''

    staff = Staff("c'4 d' e' f' g' a' b' c''")
    attach(TimeSignature((2, 4)), staff[4])
    staff._update_logical_measure_numbers()

    assert staff._logical_measure_number == 1
    assert staff[0]._logical_measure_number == 1
    assert staff[1]._logical_measure_number == 1
    assert staff[2]._logical_measure_number == 1
    assert staff[3]._logical_measure_number == 1
    assert staff[4]._logical_measure_number == 2
    assert staff[5]._logical_measure_number == 2
    assert staff[6]._logical_measure_number == 3
    assert staff[7]._logical_measure_number == 3



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__containers__nesting_01():

    target = Container([
        Container([]),
        Container([
            Container([])
        ])
    ])

    assert format(target) == stringtools.normalize(
        r'''
        {
            {
            }
            {
                {
                }
            }
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad.tools.abctools import ContextManager


class NullContextManager(ContextManager):
    r'''A context manager that does nothing.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Context managers'

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __enter__(self):
        r'''Enters context manager and does nothing.
        '''
        pass

    def __exit__(self, exc_type, exc_value, traceback):
        r'''Exits context manager and does nothing.
        '''
        pass



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Note_written_pitch_01():


    staff = Staff("d''8 e''8 f''8 g''8")
    piccolo = instrumenttools.Piccolo()
    attach(piccolo, staff)
    instrumenttools.transpose_from_sounding_pitch_to_written_pitch(staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.instrumentName = \markup { Piccolo }
            \set Staff.shortInstrumentName = \markup { Picc. }
            d'8
            e'8
            f'8
            g'8
        }
        '''
        )

    assert staff[0].written_pitch == "d'"
    assert staff[1].written_pitch == "e'"
    assert staff[2].written_pitch == "f'"
    assert staff[3].written_pitch == "g'"



# -*- coding: utf-8 -*-
from abjad.tools import selectiontools
from abjad.tools import spannertools
from abjad.tools.topleveltools import iterate
from abjad.tools.abctools import AbjadValueObject


class LogicalTieSelectorCallback(AbjadValueObject):
    r'''Logical tie selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_flatten',
        '_pitched',
        '_trivial',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        flatten=True,
        pitched=True,
        trivial=True,
        ):
        self._flatten = bool(flatten)
        self._pitched = bool(pitched)
        self._trivial = bool(trivial)

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.

        Returns tuple of selections.
        '''
        assert isinstance(argument, tuple), repr(argument)
        result = []
        if self.flatten:
            visited_logical_ties = set()
            for subexpr in argument:
                for logical_tie in self._iterate_expr(subexpr):
                    if logical_tie in visited_logical_ties:
                        continue
                    result.append(logical_tie)
                    visited_logical_ties.add(logical_tie)
        else:
            for subexpr in argument:
                subresult = []
                visited_logical_ties = set()
                for logical_tie in self._iterate_expr(subexpr):
                    if logical_tie in visited_logical_ties:
                        continue
                    subresult.append(logical_tie)
                    visited_logical_ties.add(logical_tie)
                subresult = selectiontools.Selection(subresult)
                result.append(subresult)
        return tuple(result)

    ### PRIVATE METHODS ###

    def _iterate_expr(self, argument):
        from abjad.tools import scoretools
        prototype = scoretools.Leaf
        if self.pitched:
            prototype = (scoretools.Chord, scoretools.Note)
        current_tie_spanner = None
        leaves = tuple(iterate(argument).by_class(prototype))
        for leaf in leaves:
            tie_spanners = tuple(leaf._get_spanners(spannertools.Tie))
            if not tie_spanners or \
                tie_spanners[0] is not current_tie_spanner:
                if tie_spanners:
                    current_tie_spanner = tie_spanners[0]
                else:
                    current_tie_spanner = None
                logical_tie = leaf._get_logical_tie()
                if not self.trivial and len(logical_tie) == 1:
                    continue
                yield logical_tie

    ### PUBLIC PROPERTIES ###

    @property
    def flatten(self):
        r'''Is true if callback returns all logical ties in a single selection,
        rather than grouping by each original sub-expression.

        Returns true or false.
        '''
        return self._flatten

    @property
    def pitched(self):
        r'''Is true if callback iterates pitched logical ties.

        Returns true or false.
        '''
        return self._pitched

    @property
    def trivial(self):
        r'''Is true if callback iterates trivial logical ties.

        Returns true or false.
        '''
        return self._trivial



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Component__get_in_my_logical_voice_01():

    staff = Staff("c'8 d'8 e'8 f'8")

    assert staff[0]._get_in_my_logical_voice(0, Note) is staff[0]
    assert staff[0]._get_in_my_logical_voice(1, Note) is staff[1]
    assert staff[0]._get_in_my_logical_voice(2, Note) is staff[2]
    assert staff[0]._get_in_my_logical_voice(3, Note) is staff[3]

    assert staff[3]._get_in_my_logical_voice(0, Note) is staff[3]
    assert staff[3]._get_in_my_logical_voice(-1, Note) is staff[2]
    assert staff[3]._get_in_my_logical_voice(-2, Note) is staff[1]
    assert staff[3]._get_in_my_logical_voice(-3, Note) is staff[0]


def test_scoretools_Component__get_in_my_logical_voice_02():

    staff = Staff("c'8 d'8 e'8 f'8")

    assert staff[0]._get_in_my_logical_voice(99, Note) is None


def test_scoretools_Component__get_in_my_logical_voice_03():
    r'''Different anonymous contexts create different logical voices.
    '''

    container = Container([Voice("c'8 d'8"), Voice("e'8 f'8")])
    leaves = select(container).by_leaf()

    assert leaves[0]._get_in_my_logical_voice(1, Note) is leaves[1]
    assert leaves[1]._get_in_my_logical_voice(1, Note) is None
    assert leaves[2]._get_in_my_logical_voice(1, Note) is leaves[3]
    assert container[0]._get_in_my_logical_voice(1, Voice) is None


def test_scoretools_Component__get_in_my_logical_voice_04():
    r'''Differently named contexts create different logical voices.
    '''

    container = Container([Voice("c'8 d'8"), Voice("e'8 f'8")])
    container[0].name = 'voice'
    leaves = select(container).by_leaf()

    assert container[0]._get_in_my_logical_voice(1, Voice) is None
    assert leaves[0]._get_in_my_logical_voice(1, Note) is leaves[1]
    assert leaves[1]._get_in_my_logical_voice(1, Note) is None
    assert leaves[2]._get_in_my_logical_voice(1, Note) is leaves[3]


def test_scoretools_Component__get_in_my_logical_voice_05():
    r'''Like-named contexts create the same logical voice.
    '''

    container = Container([Voice("c'8 d'8"), Voice("e'8 f'8")])
    container[0].name = 'voice'
    container[1].name = 'voice'
    leaves = select(container).by_leaf()

    assert container[0]._get_in_my_logical_voice(1, Voice) is container[1]
    assert container[1]._get_in_my_logical_voice(1, Voice) is None
    assert leaves[1]._get_in_my_logical_voice(1, Note) is leaves[2]


def test_scoretools_Component__get_in_my_logical_voice_06():
    r'''Like-named contexts create the same logical voice.
    The intervening rest exists in a different logical voice.
    '''

    container = Container([Voice("c'8 d'8"), Rest('r2'), Voice("e'8 f'8")])
    container[0].name = 'voice'
    container[-1].name = 'voice'
    leaves = select(container).by_leaf()

    assert container[0]._get_in_my_logical_voice(1, Voice) is container[2]
    assert leaves[1]._get_in_my_logical_voice(1, Note) is leaves[3]


def test_scoretools_Component__get_in_my_logical_voice_07():
    r'''Get component in same logical voice across simultaneous containers.
    '''

    container_1 = Container([Voice("c''8 d''8"), Voice("c'8 d'8")])
    container_1[0].name = 'voiceOne'
    container_1[1].name = 'voiceTwo'
    container_1.is_simultaneous = True
    container_2 = Container([Voice("e''8 f''8"), Voice("e'8 f'8")])
    container_2[0].name = 'voiceOne'
    container_2[1].name = 'voiceTwo'
    container_2.is_simultaneous = True
    staff = Staff([container_1, container_2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            <<
                \context Voice = "voiceOne" {
                    c''8
                    d''8
                }
                \context Voice = "voiceTwo" {
                    c'8
                    d'8
                }
            >>
            <<
                \context Voice = "voiceOne" {
                    e''8
                    f''8
                }
                \context Voice = "voiceTwo" {
                    e'8
                    f'8
                }
            >>
        }
        '''
        )

    assert container_1[0]._get_in_my_logical_voice(1, Voice) is container_2[0]
    assert container_1[1]._get_in_my_logical_voice(1, Voice) is container_2[1]
    assert container_1[0][1]._get_in_my_logical_voice(1, Note) is \
        container_2[0][0]
    assert container_1[1][1]._get_in_my_logical_voice(1, Note) is \
        container_2[1][0]



# -*- coding: utf-8 -*-
import abjad
from abjad.tools import lilypondfiletools


def make_lilypond_file(tuplet_duration, row_count, column_count):
    r'''Makes LilyPond file.
    '''

    score = abjad.demos.ferneyhough.make_score(
        tuplet_duration,
        row_count,
        column_count,
        )
    abjad.demos.ferneyhough.configure_score(score)
    lilypond_file = lilypondfiletools.LilyPondFile.new(score)
    abjad.demos.ferneyhough.configure_lilypond_file(lilypond_file)
    return lilypond_file


# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_agenttools_InspectionAgent_get_effective_01():
    r'''Clef defaults to none.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    for note in staff:
        clef = inspect_(note).get_effective(Clef)
        assert clef is None


def test_agenttools_InspectionAgent_get_effective_02():
    r'''Clefs carry over to notes following.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    clef = Clef('treble')
    attach(clef, staff)
    for note in staff:
        clef = inspect_(note).get_effective(Clef)
        assert clef == Clef('treble')


def test_agenttools_InspectionAgent_get_effective_03():
    r'''Clef defaults to none. Clefs carry over to notes following.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    clef = Clef('bass')
    attach(clef, staff[4])
    for i, note in enumerate(staff):
        if i in (0, 1, 2, 3):
            clef = inspect_(note).get_effective(Clef)
            assert clef is None
        else:
            clef = inspect_(note).get_effective(Clef)
            assert clef == Clef('bass')


def test_agenttools_InspectionAgent_get_effective_04():
    r'''Clefs carry over to notes following.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    clef = Clef('treble')
    attach(clef, staff[0])
    clef = Clef('bass')
    attach(clef, staff[4])
    result = [
        inspect_(note).get_effective(Clef)
        for note in staff
        ]
    clef_names = [
        'treble', 'treble', 'treble', 'treble',
        'bass', 'bass', 'bass', 'bass',
        ]
    clefs = [Clef(name) for name in clef_names]
    assert result == clefs


def test_agenttools_InspectionAgent_get_effective_05():
    r'''None cancels an explicit clef.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    clef = Clef('treble')
    attach(clef, staff[0])
    clef = Clef('bass')
    attach(clef, staff[4])
    clef = inspect_(staff[4]).get_effective(Clef)
    detach(clef, staff[4])

    for note in staff:
        clef = inspect_(note).get_effective(Clef)
        assert clef == Clef('treble')


def test_agenttools_InspectionAgent_get_effective_06():
    r'''Redudant clefs are allowed.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    clef = Clef('treble')
    attach(clef, staff[0])
    clef = Clef('treble')
    attach(clef, staff[4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \clef "treble"
            c'8
            cs'8
            d'8
            ef'8
            \clef "treble"
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_agenttools_InspectionAgent_get_effective_07():
    r'''Clefs with transposition are allowed and work as expected.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    clef = Clef('treble_8')
    attach(clef, staff[0])
    clef = Clef('treble')
    attach(clef, staff[4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \clef "treble_8"
            c'8
            cs'8
            d'8
            ef'8
            \clef "treble"
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_agenttools_InspectionAgent_get_effective_08():
    r'''Attaching and then detaching works as expected.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    clef = Clef('alto')
    attach(clef, staff[0])
    clef = inspect_(staff[0]).get_effective(Clef)
    detach(clef, staff[0])

    for leaf in staff:
        clef = inspect_(leaf).get_effective(Clef)
        assert clef is None


def test_agenttools_InspectionAgent_get_effective_09():

    staff = Staff("c'8 d'8 e'8 f'8")
    dynamic = Dynamic('f')
    attach(dynamic, staff[2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            d'8
            e'8 \f
            f'8
        }
        '''
        )

    assert inspect_(staff).get_effective(Dynamic) is None
    assert inspect_(staff[0]).get_effective(Dynamic) is None
    assert inspect_(staff[1]).get_effective(Dynamic) is None
    assert inspect_(staff[2]).get_effective(Dynamic) == Dynamic('f')
    assert inspect_(staff[3]).get_effective(Dynamic) == Dynamic('f')


def test_agenttools_InspectionAgent_get_effective_10():

    staff = Staff("c'8 d'8 e'8 f'8")
    flute = instrumenttools.Flute()
    attach(flute, staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.instrumentName = \markup { Flute }
            \set Staff.shortInstrumentName = \markup { Fl. }
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    flute = instrumenttools.Flute()
    assert inspect_(staff).get_effective(instrumenttools.Instrument) == flute
    assert inspect_(staff[0]).get_effective(instrumenttools.Instrument) == flute
    assert inspect_(staff[1]).get_effective(instrumenttools.Instrument) == flute
    assert inspect_(staff[2]).get_effective(instrumenttools.Instrument) == flute
    assert inspect_(staff[3]).get_effective(instrumenttools.Instrument) == flute


def test_agenttools_InspectionAgent_get_effective_11():
    r'''Attach key signature.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    key_signature = KeySignature('c', 'major')
    attach(key_signature, staff)

    key_signature = inspect_(staff).get_effective(KeySignature)
    assert key_signature == KeySignature('c', 'major')

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \key c \major
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_agenttools_InspectionAgent_get_effective_12():
    r'''There is no default key signature.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    key_signature = inspect_(staff).get_effective(KeySignature)
    assert key_signature is None


def test_agenttools_InspectionAgent_get_effective_13():
    r'''Attaches tempo to staff.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    tempo_1 = Tempo(Duration(1, 8), 38)
    attach(tempo_1, staff, scope=Staff)
    tempo_2 = Tempo(Duration(1, 8), 42)
    attach(tempo_2, staff[2], scope=Staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tempo 8=38
            c'8
            d'8
            \tempo 8=42
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()

    assert inspect_(staff[0]).get_effective(Tempo) == tempo_1
    assert inspect_(staff[1]).get_effective(Tempo) == tempo_1
    assert inspect_(staff[2]).get_effective(Tempo) == tempo_2
    assert inspect_(staff[3]).get_effective(Tempo) == tempo_2


def test_agenttools_InspectionAgent_get_effective_14():
    r'''Attaches tempo to chord in staff.
    '''

    staff = Staff([Chord([2, 3, 4], (1, 4))])
    tempo = Tempo(Duration(1, 8), 38)
    attach(tempo, staff[0], scope=Staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tempo 8=38
            <d' ef' e'>4
        }
        '''
        )


def test_agenttools_InspectionAgent_get_effective_15():

    staff = Staff([Note("c'4")])
    tempo = Tempo(Duration(1, 8), 38)
    attach(tempo, staff[0], scope=Staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tempo 8=38
            c'4
        }
        '''
        )


def test_agenttools_InspectionAgent_get_effective_16():
    r'''Detaches tempo.
    '''

    staff = Staff([Note("c'4")])
    tempo = Tempo(Duration(1, 8), 38)
    attach(tempo, staff[0], scope=Staff)
    detach(tempo, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'4
        }
        '''
        )


def test_agenttools_InspectionAgent_get_effective_17():
    r'''The default effective time signature is none.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")

    for leaf in staff:
        time_signature = inspect_(leaf).get_effective(TimeSignature)
        assert time_signature is None


def test_agenttools_InspectionAgent_get_effective_18():
    r'''Forced time signature settings propagate to later leaves.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    time_signature = TimeSignature((2, 8))
    attach(time_signature, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \time 2/8
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    for leaf in staff:
        time_signature = inspect_(leaf).get_effective(TimeSignature)
        assert time_signature == TimeSignature((2, 8))


def test_agenttools_InspectionAgent_get_effective_19():
    r'''Attach then detach.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    time_signature = TimeSignature((2, 8))
    attach(time_signature, staff[0])
    detach(time_signature, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    for leaf in staff:
        time_signature = inspect_(leaf).get_effective(TimeSignature)
        assert time_signature is None


def test_agenttools_InspectionAgent_get_effective_20():
    r'''Effective value of arbitrary object.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    attach('color', staff[1], scope=Staff)

    assert inspect_(staff).get_effective(str) is None
    assert inspect_(staff[0]).get_effective(str) is None
    assert inspect_(staff[1]).get_effective(str) == 'color'
    assert inspect_(staff[2]).get_effective(str) == 'color'
    assert inspect_(staff[3]).get_effective(str) == 'color'


def test_agenttools_InspectionAgent_get_effective_21():
    staff = Staff("c'8 d'8 e'8 f'8 g'8")
    attach('red', staff[0], scope=Staff)
    attach('blue', staff[2], scope=Staff)
    attach('yellow', staff[4], scope=Staff)

    assert inspect_(staff[0]).get_effective(str, n=-1) is None
    assert inspect_(staff[0]).get_effective(str, n=0) == 'red'
    assert inspect_(staff[0]).get_effective(str, n=1) is 'blue'

    assert inspect_(staff[1]).get_effective(str, n=-1) is None
    assert inspect_(staff[1]).get_effective(str, n=0) == 'red'
    assert inspect_(staff[1]).get_effective(str, n=1) == 'blue'

    assert inspect_(staff[2]).get_effective(str, n=-1) == 'red'
    assert inspect_(staff[2]).get_effective(str, n=0) == 'blue'
    assert inspect_(staff[2]).get_effective(str, n=1) == 'yellow'

    assert inspect_(staff[3]).get_effective(str, n=-1) == 'red'
    assert inspect_(staff[3]).get_effective(str, n=0) == 'blue'
    assert inspect_(staff[3]).get_effective(str, n=1) == 'yellow'

    assert inspect_(staff[4]).get_effective(str, n=-1) == 'blue'
    assert inspect_(staff[4]).get_effective(str, n=0) == 'yellow'
    assert inspect_(staff[4]).get_effective(str, n=1) is None


def test_agenttools_InspectionAgent_get_effective_22():
    staff = Staff("c'8 d'8 e'8 f'8")
    attach('red', staff[-1], scope=Staff, synthetic_offset=-1)
    attach('blue', staff[0], scope=Staff)
    assert inspect_(staff).get_effective(str) == 'blue'
    assert inspect_(staff).get_effective(str, n=-1) == 'red'



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_IndicatorExpression_name_01():
    component = Note("c'4")
    articulation = Articulation('accent', Up)
    attach(articulation, component)
    indicator_expression = inspect_(component).get_indicators(unwrap=False)[0]
    assert indicator_expression.name is None


def test_indicatortools_IndicatorExpression_name_02():
    component = Note("c'4")
    articulation = Articulation('accent', Up)
    attach(articulation, component, name='foo')
    indicator_expression = inspect_(component).get_indicators(unwrap=False)[0]
    assert indicator_expression.name == 'foo'


def test_indicatortools_IndicatorExpression_name_03():
    leaf_a = Note("c'4")
    articulation = Articulation('accent', Up)
    attach(articulation, leaf_a)
    indicator_expression_a = inspect_(leaf_a).get_indicators(unwrap=False)[0]
    assert indicator_expression_a.name is None
    leaf_b = Note("g'4")
    attach(indicator_expression_a, leaf_b)
    indicator_expression_b = inspect_(leaf_b).get_indicators(unwrap=False)[0]
    assert indicator_expression_a is not indicator_expression_b
    assert indicator_expression_b.name is None


def test_indicatortools_IndicatorExpression_name_04():
    leaf_a = Note("c'4")
    articulation = Articulation('accent', Up)
    attach(articulation, leaf_a, name='foo')
    indicator_expression_a = inspect_(leaf_a).get_indicators(unwrap=False)[0]
    assert indicator_expression_a.name == 'foo'
    leaf_b = Note("g'4")
    attach(indicator_expression_a, leaf_b)
    indicator_expression_b = inspect_(leaf_b).get_indicators(unwrap=False)[0]
    assert indicator_expression_a is not indicator_expression_b
    assert indicator_expression_b.name == 'foo'


def test_indicatortools_IndicatorExpression_name_05():
    leaf_a = Note("c'4")
    articulation = Articulation('accent', Up)
    attach(articulation, leaf_a, name='foo')
    indicator_expression_a = inspect_(leaf_a).get_indicators(unwrap=False)[0]
    assert indicator_expression_a.name == 'foo'
    leaf_b = Note("g'4")
    attach(indicator_expression_a, leaf_b, name='bar')
    indicator_expression_b = inspect_(leaf_b).get_indicators(unwrap=False)[0]
    assert indicator_expression_a is not indicator_expression_b
    assert indicator_expression_b.name == 'bar'



# -*- coding: utf-8 -*-
import os
import platform
from abjad.tools import commandlinetools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase


class Test(ScorePackageScriptTestCase):

    expected_files = [
        'test_score/test_score/segments/.gitignore',
        'test_score/test_score/segments/__init__.py',
        'test_score/test_score/segments/metadata.json',
        'test_score/test_score/segments/test_segment/__init__.py',
        'test_score/test_score/segments/test_segment/definition.py',
        ]

    if platform.system().lower() == 'windows':
        expected_files = [_.replace('/', os.path.sep) for _ in expected_files]

    def test_exists(self):
        self.create_score()
        self.create_segment('test_segment')
        with systemtools.RedirectedStreams(stdout=self.string_io):
            self.create_segment('test_segment', expect_error=True)
        self.compare_captured_output(r'''
            Creating segment subpackage 'test_segment' ...
                Path exists: test_score/segments/test_segment
        '''.replace('/', os.path.sep))

    def test_force_replace(self):
        self.create_score()
        self.create_segment('test_segment')
        with systemtools.RedirectedStreams(stdout=self.string_io):
            self.create_segment('test_segment', force=True)
        self.compare_captured_output(r'''
            Creating segment subpackage 'test_segment' ...
                Reading test_score/metadata.json ... OK!
                Reading test_score/segments/metadata.json ... OK!
                Created test_score/segments/test_segment/
        '''.replace('/', os.path.sep))

    def test_internal_path(self):
        self.create_score()
        script = commandlinetools.ManageSegmentScript()
        command = ['--new', 'test_segment']
        internal_path = self.score_path.joinpath('test_score', 'build')
        assert internal_path.exists()
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(internal_path)):
                try:
                    script(command)
                except SystemExit:
                    raise RuntimeError('SystemExit')
        self.compare_captured_output(r'''
            Creating segment subpackage 'test_segment' ...
                Reading test_score/metadata.json ... OK!
                Reading test_score/segments/metadata.json ... JSON does not exist.
                Writing test_score/segments/metadata.json
                Created test_score/segments/test_segment/
        '''.replace('/', os.path.sep))

    def test_success(self):
        self.create_score()
        script = commandlinetools.ManageSegmentScript()
        try:
            names = script._read_segments_list_json(
                self.score_path,
                verbose=False,
                )
            assert names == []
        except SystemExit:
            raise RuntimeError('SystemExit')
        command = ['--new', 'test_segment']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit:
                    raise RuntimeError('SystemExit')
        self.compare_captured_output(r'''
            Creating segment subpackage 'test_segment' ...
                Reading test_score/metadata.json ... OK!
                Reading test_score/segments/metadata.json ... JSON does not exist.
                Writing test_score/segments/metadata.json
                Created test_score/segments/test_segment/
        '''.replace('/', os.path.sep))
        assert self.segments_path.joinpath('test_segment').exists()
        self.compare_path_contents(self.segments_path, self.expected_files)
        try:
            names = script._read_segments_list_json(
                self.score_path,
                verbose=False,
                )
            assert names == ['test_segment']
        except SystemExit:
            raise RuntimeError('SystemExit')



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__misc__version_string_01():
    r'''Version strings are ignored.
    '''

    target = Staff()
    string = r'''\version "2.14.2" \new Staff { }'''
    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_FixedDurationContainer___init___01():
    r'''Initializes fixed-duration container from empty input.
    '''

    container = scoretools.FixedDurationContainer()

    assert container.target_duration == Duration(1, 4)
    assert not len(container)



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_PitchedQEvent___eq___01():

    a = quantizationtools.PitchedQEvent(1000, [0])
    b = quantizationtools.PitchedQEvent(1000, [0])

    assert a == b


def test_quantizationtools_PitchedQEvent___eq___02():

    a = quantizationtools.PitchedQEvent(1000, [0])
    b = quantizationtools.PitchedQEvent(1000, [0], ['foo', 'bar', 'baz'])
    c = quantizationtools.PitchedQEvent(9999, [0])
    d = quantizationtools.PitchedQEvent(1000, [0, 1, 4])

    assert a != b
    assert a != c
    assert a != d


def test_quantizationtools_PitchedQEvent___eq___03():

    a = quantizationtools.TerminalQEvent(100)
    b = quantizationtools.PitchedQEvent(100, [0])
    c = quantizationtools.SilentQEvent(100)

    assert a != b
    assert a != c



# -*- coding: utf-8 -*-


def all_are_positive_integer_equivalent_numbers(argument):
    '''Is true when `argument` is an iterable collection of positive
    integer-equivalent numbers. Otherwise false.

    ..  container:: example

        ::

            >>> items = [Fraction(4, 2), 2.0, 2]
            >>> mathtools.all_are_positive_integer_equivalent_numbers(items)
            True

    Returns true or false.
    '''
    from abjad.tools import mathtools
    try:
        return all(
            mathtools.is_positive_integer_equivalent_number(_)
            for _ in argument
            )
    except TypeError:
        return False



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_selectiontools_Selection__get_dominant_spanners_01():
    r'''Returns Python list of (spanner, index) pairs.
    Each (spanner, index) pair gives a spanner which dominates
    all components in list, together with the start-index
    at which spanner attaches to subelement of first
    component in list.
    Beam and trill dominate first container.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \startTrillSpan
                d'8
            }
            {
                e'8 \glissando
                f'8 ] \glissando
            }
            {
                g'8 \glissando
                a'8 \stopTrillSpan
            }
        }
        '''
        )

    receipt = voice[:1]._get_dominant_spanners()

    assert len(receipt) == 2
    assert (beam, 0) in receipt
    assert (trill, 0) in receipt


def test_selectiontools_Selection__get_dominant_spanners_02():
    r'''Beam, glissando and trill all dominate second container.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    receipt = voice[1:2]._get_dominant_spanners()

    assert len(receipt) == 3
    assert (beam, 2) in receipt
    assert (glissando, 0) in receipt
    assert (trill, 2) in receipt


def test_selectiontools_Selection__get_dominant_spanners_03():
    r'''Glissando and trill dominate last container.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    receipt = voice[-1:]._get_dominant_spanners()

    assert len(receipt) == 2
    assert (glissando, 2) in receipt
    assert (trill, 4) in receipt


def test_selectiontools_Selection__get_dominant_spanners_04():
    r'''Beam and trill dominate first two containers.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    receipt= voice[:2]._get_dominant_spanners()

    assert len(receipt) == 2
    assert (beam, 0) in receipt
    assert (trill, 0) in receipt


def test_selectiontools_Selection__get_dominant_spanners_05():
    r'''Glissando and trill dominate last two containers.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    receipt = voice[-2:]._get_dominant_spanners()

    assert len(receipt) == 2
    assert (glissando, 0) in receipt
    assert (trill, 2) in receipt


def test_selectiontools_Selection__get_dominant_spanners_06():
    r'''Only trill dominates all three containers.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    receipt = voice[:]._get_dominant_spanners()

    assert len(receipt) == 1
    assert (trill, 0) in receipt


def test_selectiontools_Selection__get_dominant_spanners_07():
    r'''Only trill dominates voice.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    receipt = select(voice)._get_dominant_spanners()

    assert len(receipt) == 1
    assert (trill, 0) in receipt


def test_selectiontools_Selection__get_dominant_spanners_08():
    r'''Only trill dominates first two notes.
    Note that trill attaches to notes.
    Note that beam and glissando attach to containers.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    receipt = select(leaves[:2])._get_dominant_spanners()

    assert len(receipt) == 2
    assert (beam, 0) in receipt
    assert (trill, 0) in receipt



# -*- coding: utf-8 -*-
from abjad import *


def test_metertools_Meter___init___01():

    time_signature = TimeSignature((2, 4))
    assert metertools.Meter(time_signature).rtm_format == \
        '(2/4 (1/4 1/4))'


def test_metertools_Meter___init___02():

    time_signature = TimeSignature((3, 4))
    assert metertools.Meter(time_signature).rtm_format == \
        '(3/4 (1/4 1/4 1/4))'


def test_metertools_Meter___init___03():

    time_signature = TimeSignature((4, 4))
    assert metertools.Meter(time_signature).rtm_format == \
        '(4/4 (1/4 1/4 1/4 1/4))'


def test_metertools_Meter___init___04():

    time_signature = TimeSignature((6, 8))
    assert metertools.Meter(time_signature).rtm_format == \
        '(6/8 ((3/8 (1/8 1/8 1/8)) (3/8 (1/8 1/8 1/8))))'


def test_metertools_Meter___init___05():

    time_signature = TimeSignature((5, 8))
    assert metertools.Meter(time_signature).rtm_format == \
        '(5/8 ((3/8 (1/8 1/8 1/8)) (2/8 (1/8 1/8))))'


def test_metertools_Meter___init___06():

    time_signature = TimeSignature((12, 4))
    assert metertools.Meter(time_signature).rtm_format == \
        '(12/4 ((3/4 (1/4 1/4 1/4)) (3/4 (1/4 1/4 1/4)) (3/4 (1/4 1/4 1/4)) (3/4 (1/4 1/4 1/4))))'


def test_metertools_Meter___init___07():

    time_signature = TimeSignature((1, 4))
    assert metertools.Meter(time_signature).rtm_format == \
        '(1/4 (1/4))'


def test_metertools_Meter___init___08():

    time_signature = TimeSignature((10, 4))
    assert metertools.Meter(time_signature).rtm_format == \
        '(10/4 ((5/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)))) (5/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4))))))'

def test_metertools_Meter___init___09():

    time_signature = TimeSignature((11, 4))
    assert metertools.Meter(time_signature).rtm_format == \
        '(11/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)) (2/4 (1/4 1/4)) (2/4 (1/4 1/4)) (2/4 (1/4 1/4))))'


def test_metertools_Meter___init___10():

    hierarchy = metertools.Meter('(4/4 ((2/4 (1/4 1/4)) (2/4 (1/4 1/4))))')



# -*- coding: utf-8 -*-


def sign(n):
    r'''Gets sign of `n`.
    
    ..  container:: example

        Returns -1 on negative `n`:

        ::

            >>> mathtools.sign(-96.2)
            -1

        Returns 0 when `n` is 0:

        ::

            >>> mathtools.sign(0)
            0

        Returns 1 on positive `n`:

        ::

            >>> mathtools.sign(Duration(9, 8))
            1

    Returns -1, 0 or 1.
    '''
    return (0 < n) - (n < 0)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.demos.ferneyhough.make_lilypond_file import make_lilypond_file


def test_demos_ferneyhough_01():

    lilypond_file = make_lilypond_file(Duration(1, 4), 10, 6)



# -*- coding: utf-8 -*-
from abjad.tools import systemtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class BarLine(AbjadValueObject):
    r'''Bar line.

    ..  container:: example

        **Example 1.** Final bar line:

        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> bar_line = indicatortools.BarLine('|.')
            >>> attach(bar_line, staff[-1])
            >>> show(staff) # doctest: +SKIP

        ::

            >>> bar_line
            BarLine('|.')

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                c'4
                d'4
                e'4
                f'4
                \bar "|."
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_abbreviation',
        '_default_scope',
        )

    _format_slot = 'closing'

    ### INITIALIZER ##

    def __init__(self, abbreviation='|'):
        from abjad.tools import scoretools
        assert isinstance(abbreviation, str), repr(abbreviation)
        self._abbreviation = abbreviation
        self._default_scope = scoretools.Staff

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return repr(self.abbreviation)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            storage_format_is_indented=False,
            storage_format_args_values=[self.abbreviation],
            )

    def _get_lilypond_format(self):
        return r'\bar "{}"'.format(self.abbreviation)

    ## PUBLIC PROPERTIES ##

    @property
    def abbreviation(self):
        r'''Gets abbreviation of bar line.

        ..  container:: example

            ::

                >>> bar_line = indicatortools.BarLine('|.')
                >>> bar_line.abbreviation
                '|.'

        Returns string.
        '''
        return self._abbreviation

    @property
    def default_scope(self):
        r'''Gets default scope of bar line.

        ..  container:: example

            ::

                >>> bar_line = indicatortools.BarLine('|.')
                >>> bar_line.default_scope
                <class 'abjad.tools.scoretools.Staff.Staff'>

        Bar lines are scoped to the staff by default.

        Returns staff.
        '''
        return self._default_scope



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_datastructuretools_TreeContainer_index_01():

    leaf_a = datastructuretools.TreeNode()
    leaf_b = datastructuretools.TreeNode()
    leaf_c = datastructuretools.TreeNode()
    subcontainer = datastructuretools.TreeContainer([leaf_b, leaf_c])
    leaf_d = datastructuretools.TreeNode()
    container = datastructuretools.TreeContainer([
        leaf_a,
        subcontainer,
        leaf_d,
        ])

    assert container.index(leaf_a) == 0
    assert container.index(subcontainer) == 1
    assert container.index(leaf_d) == 2

    pytest.raises(ValueError, 'container.index(leaf_b)')
    pytest.raises(ValueError, 'container.index(leaf_c)')



# -*- coding: utf-8 -*-
import os
import shutil
import subprocess
import tempfile


class Graph(object):
    r'''IPython replacement callable for `topleveltools.graph()`.
    '''

    ### SPECIAL METHODS ###

    def __call__(
        self,
        argument,
        layout='dot',
        graph_attributes=None,
        node_attributes=None,
        edge_attributes=None,
        **keywords
        ):
        r'''A replacement for Ajbad's graph function for IPython Notebook.
        '''
        from abjad.tools import systemtools
        from IPython.core.display import display_png

        if isinstance(argument, str):
            graphviz_format = argument
        else:
            assert hasattr(argument, '__graph__')
            graphviz_graph = argument.__graph__(**keywords)
            if graph_attributes:
                graphviz_graph.attributes.update(graph_attributes)
            if node_attributes:
                graphviz_graph.node_attributes.update(node_attributes)
            if edge_attributes:
                graphviz_graph.edge_attributes.update(edge_attributes)
            graphviz_format = str(graphviz_graph)

        valid_layouts = (
            'circo',
            'dot',
            'fdp',
            'neato',
            'osage',
            'sfdp',
            'twopi',
            )
        assert layout in valid_layouts

        message = 'cannot find `{}` command-line tool.'
        message = message.format(layout)
        message += ' Please download Graphviz from graphviz.org.'
        assert systemtools.IOManager.find_executable(layout), message
        assert systemtools.IOManager.find_executable('convert')

        temporary_directory = tempfile.mkdtemp()
        dot_path = os.path.join(temporary_directory, 'graph.dot')
        pdf_path = os.path.join(temporary_directory, 'graph.pdf')
        png_path = os.path.join(temporary_directory, 'graph.png')

        with open(dot_path, 'w') as file_pointer:
            file_pointer.write(graphviz_format)

        command = '{} -v -Tpdf {} -o {}'
        command = command.format(layout, dot_path, pdf_path)
        subprocess.call(command, shell=True)

        command = 'convert {} -trim {}'
        command = command.format(pdf_path, png_path)
        subprocess.call(command, shell=True)

        systemtools.IOManager.spawn_subprocess(command)
        with open(png_path, 'rb') as file_pointer:
            png = file_pointer.read()
        shutil.rmtree(temporary_directory)
        display_png(png, raw=True)



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.topleveltools import iterate


def iterate_out_of_range_notes_and_chords(argument):
    '''Iterates notes and chords in `argument` outside traditional instrument
    ranges.

    ..  todo:: Move to WellformednessManager.

    ..  container:: example

        **Example.** Iterates out-of-range notes and chords:

        ::

            >>> staff = Staff("c'8 r8 <d fs>8 r8")
            >>> violin = instrumenttools.Violin()
            >>> attach(violin, staff)
            >>> show(staff) # doctest: +SKIP

        ::

            >>> list(
            ... instrumenttools.iterate_out_of_range_notes_and_chords(
            ... staff))
            [Chord('<d fs>8')]

    Returns generator.
    '''
    from abjad.tools import instrumenttools
    prototype = (scoretools.Note, scoretools.Chord)
    for note_or_chord in iterate(argument).by_class(prototype):
        instrument = note_or_chord._get_effective(
            instrumenttools.Instrument)
        if instrument is None:
            message = 'no instrument found.'
            raise ValueError(message)
        if note_or_chord not in instrument.pitch_range:
            yield note_or_chord


# -*- coding: utf-8 -*-
import collections


def _flatten_sequence(sequence, classes=None, depth=-1, indices=None):
    r'''DEPRECATED. Use Sequence.flatten() instead.
    '''
    from abjad.tools import selectiontools
    from abjad.tools import sequencetools
    if sequence is None:
        callback = sequencetools.FlattenCallback(
            classes=classes,
            depth=depth,
            indices=indices,
            )
        return callback
    if classes is None:
        classes = (collections.Sequence, selectiontools.Selection)
    if not isinstance(sequence, collections.Sequence):
        message = 'must be sequence or selection: {!r}.'
        message = message.format(sequence)
        raise Exception(message)
    sequence_type = type(sequence)
    if indices is None:
        return sequence_type(_flatten_helper(sequence, classes, depth))
    else:
        return sequence_type(
            _flatten_at_indices_helper(sequence, indices, classes, depth)
            )

# creates an iterator that can generate a flattened list,
# descending down into child elements to a depth given in the arguments.
# note that depth < 0 is effectively equivalent to infinity.
def _flatten_helper(sequence, classes, depth):
    if not isinstance(sequence, classes):
        yield sequence
    elif depth == 0:
        for i in sequence:
            yield i
    else:
        for i in sequence:
            # flatten an iterable by one level
            for j in _flatten_helper(i, classes, depth - 1):
                yield j

def _flatten_at_indices_helper(sequence, indices, classes, depth):
    from abjad.tools import sequencetools
    if classes is None:
        classes = (list, tuple)
    if not isinstance(sequence, classes):
        raise TypeError()
    ltype = type(sequence)
    len_l = len(sequence)
    indices = [x if 0 <= x else len_l + x for x in indices]
    result = []
    for i, element in enumerate(sequence):
        if i in indices:
            try:
                flattened = sequencetools.flatten_sequence(
                    element,
                    classes=classes,
                    depth=depth,
                    )
                result.extend(flattened)
            except:
                result.append(element)
        else:
            result.append(element)
    result = ltype(result)
    return result



# -*- coding: utf-8 -*-


def is_nonnegative_integer_equivalent_number(argument):
    '''Is true when `argument` is a nonnegative integer-equivalent number.
    Otherwise false.

    ..  container:: example

        ::

            >>> duration = Duration(4, 2)
            >>> mathtools.is_nonnegative_integer_equivalent_number(duration)
            True

    Returns true or false.
    '''
    from abjad.tools import mathtools
    return mathtools.is_integer_equivalent_number(argument) and 0 <= argument



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TreeContainer___len___01():

    leaf_a = datastructuretools.TreeNode()
    leaf_b = datastructuretools.TreeNode()
    leaf_c = datastructuretools.TreeNode()
    subcontainer = datastructuretools.TreeContainer([leaf_b, leaf_c])
    leaf_d = datastructuretools.TreeNode()

    container = datastructuretools.TreeContainer([
        leaf_a,
        subcontainer,
        leaf_d,
        ])

    assert len(container) == 3



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_RomanNumeral_suspension_01():

    roman_numeral = tonalanalysistools.RomanNumeral(5, 'major', 5, 0, (4, 3))

    assert roman_numeral.suspension == tonalanalysistools.ChordSuspension(4, 3)
    assert roman_numeral.suspension.start == tonalanalysistools.ScaleDegree(4)
    assert roman_numeral.suspension.stop == tonalanalysistools.ScaleDegree(3)



# -*- coding: utf-8 -*-
import os
import shutil
import tempfile


class Show(object):
    r'''IPython replacement callable for `topleveltools.show()`.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, argument):
        r'''A replacement for Ajbad's show function for IPython Notebook.
        '''
        from abjad.tools import systemtools
        from abjad.tools import topleveltools
        from IPython.core.display import display_png
        assert systemtools.IOManager.find_executable('lilypond')
        assert systemtools.IOManager.find_executable('convert')
        assert hasattr(argument, '__illustrate__')
        temporary_directory = tempfile.mkdtemp()
        temporary_file_path = os.path.join(
            temporary_directory,
            'output.png',
            )
        result = topleveltools.persist(argument).as_png(temporary_file_path)
        pngs = []
        for file_path in result[0]:
            command = 'convert {file_path} -trim {file_path}'
            command = command.format(file_path=file_path)
            systemtools.IOManager.spawn_subprocess(command)
            with open(file_path, 'rb') as file_pointer:
                file_contents = file_pointer.read()
                pngs.append(file_contents)
        shutil.rmtree(temporary_directory)
        for png in pngs:
            display_png(png, raw=True)



# -*- coding: utf-8 -*-
from abjad.tools import sequencetools
from abjad.tools import datastructuretools
from abjad.tools import selectiontools
from abjad.tools.abctools import AbjadValueObject


class CountsSelectorCallback(AbjadValueObject):
    r'''Counts selector callback.

    ..  container:: example

        **Example 1.** Initializes callback by hand:

        ::

            >>> callback = selectortools.CountsSelectorCallback([3])
            >>> f(callback)
            selectortools.CountsSelectorCallback(
                counts=datastructuretools.CyclicTuple(
                    [3]
                    ),
                cyclic=True,
                fuse_overhang=False,
                nonempty=False,
                overhang=True,
                rotate=True,
                )

    ..  container:: example

        **Example 2.** Selects components:

        ::

            >>> selector = selectortools.Selector()
            >>> selector = selector.by_leaf()
            >>> selector = selector.by_counts([3])
            >>> staff = Staff("c'8 r8 d'8 e'8 r8 f'8 g'8 a'8 b'8 r8 c''8")
            >>> selector(staff)
            Selection([Selection([Note("c'8"), Rest('r8'), Note("d'8")])])

    ..  container:: example

        **Example 3.** Selects objects:

        ::

            >>> selector = selectortools.Selector()
            >>> selector = selector.by_counts([3])
            >>> numbers = [1, 'two', 'three', 4, -5, 'foo', 7.0, 8]
            >>> selector(numbers)
            Selection([Selection([1, 'two', 'three'])])

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_counts',
        '_cyclic',
        '_fuse_overhang',
        '_overhang',
        '_rotate',
        '_nonempty',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        counts=(3,),
        cyclic=True,
        fuse_overhang=False,
        nonempty=False,
        overhang=True,
        rotate=True,
        ):
        counts = datastructuretools.CyclicTuple(int(_) for _ in counts)
        self._counts = counts
        self._cyclic = bool(cyclic)
        self._fuse_overhang = bool(fuse_overhang)
        self._overhang = bool(overhang)
        self._rotate = bool(rotate)
        self._nonempty = bool(nonempty)

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.

        Returns tuple in which each item is a selection or component.
        '''
        assert isinstance(argument, tuple), repr(argument)
        if rotation is None:
            rotation = 0
        rotation = int(rotation)
        result = []
        counts = self.counts
        if self.rotate:
            counts = sequencetools.Sequence(counts).rotate(n=-rotation)
            counts = datastructuretools.CyclicTuple(counts)
        for subexpr in argument:
            groups = sequencetools.Sequence(subexpr).partition_by_counts(
                [abs(_) for _ in counts],
                cyclic=self.cyclic,
                overhang=self.overhang,
                )
            groups = list(groups)
            if self.overhang and self.fuse_overhang and 1 < len(groups):
                last_count = counts[(len(groups) - 1) % len(counts)]
                if len(groups[-1]) != last_count:
                    last_group = groups.pop()
                    groups[-1] += last_group
            subresult = []
            for i, group in enumerate(groups):
                try:
                    count = counts[i]
                except:
                    raise Exception(counts, i)
                if count < 0:
                    continue
                items = selectiontools.Selection(group)
                subresult.append(items)
            if self.nonempty and not subresult:
                group = selectiontools.Selection(groups[0])
                subresult.append(group)
            result.extend(subresult)
            if self.rotate:
                counts = sequencetools.Sequence(counts).rotate(n=-1)
                counts = datastructuretools.CyclicTuple(counts)
        return tuple(result)

    ### PUBLIC PROPERTIES ###

    @property
    def counts(self):
        r'''Gets counts selector callback counts.

        Returns tuple.
        '''
        return self._counts

    @property
    def cyclic(self):
        r'''Gets counts selector callback cyclicity.

        Returns true or false.
        '''
        return self._cyclic

    @property
    def fuse_overhang(self):
        r'''Gets counts selector callback fuse overhang flag.

        Returns ordinal constant.
        '''
        return self._fuse_overhang

    @property
    def nonempty(self):
        r'''Gets counts selector callback nonempty flag.

        Returns true or false.
        '''
        return self._nonempty

    @property
    def overhang(self):
        r'''Gets counts selector callback overhang flag.

        Returns true or false.
        '''
        return self._overhang

    @property
    def rotate(self):
        r'''Gets counts selector callback rotate flag.

        Returns true or false.
        '''
        return self._rotate



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHead_is_cautionary_01():

    note_head = scoretools.NoteHead(written_pitch="c'")
    assert note_head.is_cautionary is None
    note_head.is_cautionary = True
    assert note_head.is_cautionary == True
    note_head.is_cautionary = False
    assert note_head.is_cautionary == False
